(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * squareroot_test.thy --- testing integer squareroots
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2007, ETH Zurich, Switzerland
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: squareroot_test.thy 7260 2008-02-10 22:27:50Z brucker $ *)

(* ML {* add_path "$ISABELLE_HOME/src/HOL/IMP"; *} *)

theory 
  squareroot_test
imports
  hoare_test_theory
begin

section{* The Definition of the Integer-Squareroot Program*}
constdefs  
  squareroot :: "[loc,loc,loc,loc] => com"
  "squareroot tm sum i a ==
   ((  tm   :== (\<lambda>s. 1));
    (( sum  :== (\<lambda>s. 1));
     ((i    :== (\<lambda>s. 0));
       WHILE (\<lambda>s. (s sum) <= (s a)) DO
         (( i    :== (\<lambda>s. (s i) + 1));
          ((tm   :== (\<lambda>s. (s tm) + 2));
           (sum  :== (\<lambda>s. (s tm) + (s sum)))))))
   )"

constdefs
  pre   :: assn
  "pre  \<equiv>  \<lambda> x. True"
  post  :: "[loc,loc] \<Rightarrow> assn"
  "post a i \<equiv>   \<lambda> s. (s i)*(s i)\<le>(s a) \<and> s a < (s i + 1)*(s i + 1)"
  inv   :: "[loc,loc,loc,loc] \<Rightarrow> assn"
  "inv i sum tm a \<equiv> \<lambda>s.(s i + 1) * (s i + 1) = s sum 
                       \<and> s tm = (2 * (s i) + 1) 
                       \<and> (s i) * (s i) <= (s a)"



section{* Computing Program Paths and their Path-Constraints*}


lemma derive_pathconds: 
  assumes no_alias : "sum \<noteq> i \<and> i \<noteq> sum \<and> tm \<noteq> sum  \<and> 
                      sum \<noteq> tm \<and> sum \<noteq> a \<and> a \<noteq> sum \<and>
                      tm \<noteq> i \<and> i \<noteq> tm \<and> tm \<noteq> a \<and> a \<noteq> tm  \<and> 
                      a \<noteq> i \<and> i \<noteq> a"
  shows  "\<langle>unwind(3, squareroot tm sum i a), s\<rangle> \<longrightarrow>\<^sub>c s'"

  apply(simp add: squareroot_def uw_while)
  apply(rule If_split, simp_all add: update_def no_alias)+
txt{* The resulting proof state capturing the test hypothesis
  as well as the resulting 4 evaluation paths (no entry into
  loop, 1 pass, 2 passes and 3 passes through the loop) looks as 
  follows:
  @{subgoals[display]}
  *}
  oops

(* goal (lemma derive_test_cases, 4 subgoals):
 1. Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc 0)))))))) \<le> s a \<Longrightarrow>
    \<langle>WHILE \<lambda>s. s sum
               \<le> s a DO i :== \<lambda>s. Suc (s
  i) ; (tm :== \<lambda>s. Suc (Suc (s tm)) ; sum :== \<lambda>s. s tm + s sum ),s
    (i := Suc (Suc (Suc 0)),
     tm := Suc (Suc (Suc (Suc (Suc (Suc (Suc 0)))))),
     sum :=
       Suc (Suc (Suc (Suc (Suc (Suc (Suc
(Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc 0))))))))))))))))\<rangle>
    \<longrightarrow>\<^sub>c s'
 2. \<lbrakk>Suc (Suc (Suc (Suc 0))) \<le> s a;
     \<not> Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc 0)))))))) \<le> s a\<rbrakk>
    \<Longrightarrow> s' = s
       (i := Suc (Suc 0), tm := Suc (Suc (Suc (Suc (Suc 0)))),
        sum := Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc 0)))))))))
 3. \<lbrakk>Suc 0 \<le> s a; \<not> Suc (Suc (Suc (Suc 0))) \<le> s a\<rbrakk>
    \<Longrightarrow> s' = s
       (i := Suc 0, tm := Suc (Suc (Suc 0)), sum := Suc (Suc (Suc (Suc 0))))
 4. \<not> Suc 0 \<le> s a \<Longrightarrow> s' = s(tm := Suc 0, sum := Suc 0, i := 0)
 *)


text{* \textbf{Summary}: With this approach, one can synthesize paths 
       and their conditions. *}


section{* Testing Specifications *}
 

lemma wb_specification_test: 
  assumes no_alias : "sum \<noteq> i \<and> i \<noteq> sum \<and> tm \<noteq> sum  \<and> 
                      sum \<noteq> tm \<and> sum \<noteq> a \<and> a \<noteq> sum \<and>
                      tm \<noteq> i \<and> i \<noteq> tm \<and> tm \<noteq> a \<and> a \<noteq> tm  \<and> 
                      a \<noteq> i \<and> i \<noteq> a"
  shows  "|- {pre} squareroot tm sum i a {post a i}"
  
  apply(simp add: squareroot_def pre_def)
  apply(rule_tac n1 = "3" in iffD2[OF setup_test])
  apply(rule allI, rule allI, simp, rule impI) 
  apply(simp_all add: update_def no_alias uw_while)
  apply(erule If_splitE, simp_all add: update_def no_alias uw_while)+
txt{* We ``wrap'' the first part into the test hypothesis: *}
  apply(erule rev_mp) back
  apply(erule THYP_app1_rev)

txt{* The resulting proof state captures the essence of this
      white box test:

    @{subgoals[display]}
  
   Now testing all paths for compliance to post condition: *}
   apply(simp_all add: no_alias post_def)

txt{* In this special case---arithmetic constraints---the system can
   even \textbf{verify} these constraints, i.e. the simplifier shows
   that all postconditions follow from the initial constraints
   and the computed relation between pre-state and post state.

    @{subgoals[display]}

   To say it loud and clearly: The white box test decomposes 
   the original specification into a
   test hypothesis for cases with $3^3 = 9 <= s a$ and all other
   cases (e.g. $2^2 = 4 <= s a \land s a < 9$). The latter have been 
   proven automatically. *}
  
(*
 1. \<And>s t. THYP
           (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc 0)))))))) \<le> s a \<longrightarrow>
            \<langle>WHILE \<lambda>s. s sum
                       \<le> s a DO i :== \<lambda>s.
   Suc (s i) ; (tm :== \<lambda>s. Suc (Suc (s tm)) ; sum :== \<lambda>s. s tm + s sum ),s
            (i := Suc (Suc (Suc 0)),
             tm := Suc (Suc (Suc (Suc (Suc (Suc (Suc 0)))))),
             sum :=
               Suc (Suc (Suc (Suc (Suc (Suc
   (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc (Suc 0))))))))))))))))\<rangle>
            \<longrightarrow>\<^sub>c t \<longrightarrow> t i * t i \<le> t a \<and> t a < 
                            Suc (t i + (t i + t i * t i))) *)
  oops
 
section{* An Alternative Approach with an On-The-Fly generated Explicit Test-Hyp.*}


lemma path_exploration_test: 
  assumes no_alias : "sum \<noteq> i \<and> i \<noteq> sum \<and> tm \<noteq> sum  \<and> 
                      sum \<noteq> tm \<and> sum \<noteq> a \<and> a \<noteq> sum \<and>
                      tm \<noteq> i \<and> i \<noteq> tm \<and> tm \<noteq> a \<and> a \<noteq> tm  \<and> 
                      a \<noteq> i \<and> i \<noteq> a"
  shows  "|- {pre} squareroot tm sum i a {post a i}"
  txt{* We fire the basic white-box scenario: *}
  apply (rule wp_test [of _ "3"])
  txt{* Given the concrete unwinding factor and the concrete program
        term, standard normalization yields an "Path Exhaustion Theorem"
        with the explicit test hypothesis: *}
  apply(auto simp: squareroot_def update_def no_alias uw_while)
  apply(tactic "thyp_ify 1")
  defer 1
  txt {* and we reach the following instantiation of a white-box 
         test-theorem (with explicit test-hypothesis for the uncovered paths): 
         @{subgoals [display, goals_limit=50]}
         Now we allso perform the "tests" by symbolic execution: *}
  apply(auto simp: no_alias pre_def post_def)
  txt {* which leaves us just with test-hypothesis case; for all
         paths \emph{not} leading to a remaining while, the program
         is correct.
          @{subgoals [display, goals_limit=50]}
      *}

  oops


end
 
