theory DMZDatatype
imports Main PortCombinators "../../src/FWTesting" 
begin

(*
datatype DMZ = Mail | Web | Other_dmz
*)

datatype Networks = Intranet | Internet | Mail | Web | DMZ

instance Networks::net .. 

constdefs
  intranet::"Networks subnet"
  "intranet \<equiv> {{a. a= Intranet}}"
  dmz :: "Networks subnet"
  "dmz \<equiv> {{a. a= DMZ}}"
  mail :: "Networks subnet"
  "mail \<equiv> {{a. a=Mail}}"
  web :: "Networks subnet"
  "web \<equiv> {{a.  a=Web}}"

  internet :: "Networks subnet"
  "internet \<equiv> {{a. a= Internet}}"



constdefs 
  Intranet_mail :: "(Networks,'b) Rule"
  "Intranet_mail \<equiv> (allow_prots_from_to {smtp,imap} intranet mail)" 

  Intranet_Internet :: "(Networks,'b) Rule"
  "Intranet_Internet \<equiv> allow_prots_from_to {http,https} intranet internet"
     
  Internet_web :: "(Networks,'b) Rule"
  "Internet_web \<equiv> (allow_prots_from_to {http,https} internet web)"
          
  Internet_mail :: "(Networks,'b) Rule"
  "Internet_mail \<equiv>  (allow_prot_from_to smtp internet dmz)"


constdefs 
  policyProtocol :: "(Networks, DummyContent) Policy"
 "policyProtocol \<equiv> deny_all ++ 
                  Intranet_Internet ++ 
                  Intranet_mail ++  
                  Internet_mail ++ 
                  Internet_web"



text {* We only want to create test cases which are sent between the three main networks --- e.g. not between the mailserver and the dmz. Therefore, the constraint looks as follows. *}

constdefs
 not_in_same_net :: "(Networks,DummyContent) packet \<Rightarrow> bool"
 "not_in_same_net x  \<equiv>  (src x \<sqsubset> internet \<longrightarrow> \<not> dest x \<sqsubset> internet) \<and> 
                        (src x \<sqsubset> intranet \<longrightarrow> \<not> dest x \<sqsubset> intranet) \<and> 
                        (src x \<sqsubset> dmz \<longrightarrow> \<not> dest x \<sqsubset> dmz) \<and> 
                        (src x \<sqsubset> mail \<longrightarrow> ((dest x \<sqsubset> internet) \<or> (dest x \<sqsubset> intranet))) \<and> 
                        (src x \<sqsubset> web \<longrightarrow> ((dest x \<sqsubset> internet) \<or> (dest x \<sqsubset> intranet)))"

lemmas PolicyLemmas =  policyProtocol_def  dmz_def internet_def intranet_def mail_def web_def 
                       Intranet_Internet_def Intranet_mail_def 
                       Internet_web_def Internet_mail_def



test_spec "not_in_same_net x \<longrightarrow> FUT (x:: (Networks,DummyContent) packet) = policyProtocol x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas src_def dest_def in_subnet_def protocol_def stateless_rules )
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def src_def dest_def in_subnet_def PolicyLemmas)
store_test_thm "protocol"

gen_test_data "protocol"



end