theory DMZWithoutPorts
imports Main IntegerNetwork PortCombinators "../../src/FWTesting" 
begin

text{* This is the fourth scenario, slightly more complicated than the previous one, as we now also model specific servers within one network. Therefore, we cannot use anymore the modelling using datatypes, but only use the one where an address is modelled as an integer (with ports).

The scenario is the following:

\begin{labeling}{Networks:}
\item[Networks:]
  \begin{itemize}
  \item Intranet (Company intern network)
  \item DMZ (demilitarised zone, servers, etc), containing
     at least two distinct servers ``mail'' and ``web''
  \item Internet (``all others'') 
  \end{itemize}
\item[Policy:]
  \begin{itemize}
  \item allow http(s) from Intranet to Internet
  \item deny all trafic from Internet to Intranet
  \item allo imaps and smtp from intranet to mailserver
  \item allow smtp from Internet to mailserver
  \item allow http(s) from Internet to webserver 
  \item deny everything else
  \end{itemize}
  
\end{labeling}
*}


constdefs
   intranet::"IntegerNetwork subnet"
   "intranet \<equiv> {{a . (a > 1 \<and> a < 4) }}"

  dmz :: "IntegerNetwork subnet"
  "dmz \<equiv> {{a. (a > 6) \<and> (a < 11)}}"
  mail :: "IntegerNetwork subnet"
  "mail \<equiv> {{a. a = 7}}"
  web :: "IntegerNetwork subnet"
  "web \<equiv> {{a. a = 8 }}"

  internet :: "IntegerNetwork subnet"
  "internet \<equiv> {{a.  \<not> ( (a > 1 \<and> a < 4) \<or> (a > 6) \<and> (a < 11))   }}"

constdefs 
  Intranet_mail :: "(IntegerNetwork,'b) Rule"
  "Intranet_mail \<equiv> (allow_prots_from_to {smtp,imap} intranet mail)" 

  Intranet_Internet :: "(IntegerNetwork,'b) Rule"
  "Intranet_Internet \<equiv> allow_prots_from_to {http,https} intranet internet"
     
  Internet_web :: "(IntegerNetwork,'b) Rule"
  "Internet_web \<equiv> (allow_prots_from_to {http,https} internet web)"
          
  Internet_mail :: "(IntegerNetwork,'b) Rule"
  "Internet_mail \<equiv>  (allow_prot_from_to smtp internet dmz)"




constdefs 
  policyProtocol :: "(IntegerNetwork, DummyContent) Policy"
 "policyProtocol \<equiv> deny_all ++ 
                  Intranet_Internet ++ 
                  Intranet_mail ++  
                  Internet_mail ++ 
                  Internet_web"



text {* We only want to create test cases which are sent between the three main networks --- e.g. not between the mailserver and the dmz. Therefore, the constraint looks as follows. *}

constdefs
 not_in_same_net :: "(IntegerNetwork,DummyContent) packet \<Rightarrow> bool"
 "not_in_same_net x  \<equiv>  (src x \<sqsubset> internet \<longrightarrow> \<not> dest x \<sqsubset> internet) \<and> 
                        (src x \<sqsubset> intranet \<longrightarrow> dest x \<sqsubset> intranet) \<and> 
                        (src x \<sqsubset> dmz \<longrightarrow> dest x \<sqsubset> dmz)"

lemmas PolicyLemmas =  policyProtocol_def  dmz_def internet_def intranet_def mail_def web_def 
                       Intranet_Internet_def Intranet_mail_def 
                       Internet_web_def Internet_mail_def

testgen_params[iterations=100]


test_spec "not_in_same_net x \<longrightarrow> FUT (x:: (IntegerNetwork,DummyContent) packet) = policyProtocol x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas IntegerLemmas PortCombinators stateless_rules src_def dest_def in_subnet_def)
apply (gen_test_cases "FUT")
apply simp_all
apply (simp_all add: not_in_same_net_def src_def dest_def in_subnet_def PolicyLemmas src_def dest_def in_subnet_def)
store_test_thm "test"

gen_test_data "test"


thm test.test_data


end