theory DMZProtocol
imports Main IntegerPairNetwork PortCombinators "../../src/FWTesting" 
begin

text{* This is the fourth scenario, slightly more complicated than the previous one, as we now also model specific servers within one network. Therefore, we cannot use anymore the modelling using datatypes, but only use the one where an address is modelled as an integer (with ports).

The scenario is the following:

\begin{labeling}{Networks:}
\item[Networks:]
  \begin{itemize}
  \item Intranet (Company intern network)
  \item DMZ (demilitarised zone, servers, etc), containing
     at least two distinct servers ``mail'' and ``web''
  \item Internet (``all others'') 
  \end{itemize}
\item[Policy:]
  \begin{itemize}
  \item allow http(s) from Intranet to Internet
  \item deny all trafic from Internet to Intranet
  \item allo imaps and smtp from intranet to mailserver
  \item allow smtp from Internet to mailserver
  \item allow http(s) from Internet to webserver 
  \item deny everything else
  \end{itemize}
  
\end{labeling}
*}


constdefs
   intranet::"IntegerPair subnet"
   "intranet \<equiv> {{(a,b) . (a > 1 \<and> a < 4) }}"

  dmz :: "IntegerPair subnet"
  "dmz \<equiv> {{(a,b). (a > 6) \<and> (a < 11)}}"
  mail :: "IntegerPair subnet"
  "mail \<equiv> {{(a,b). a = 7}}"
  web :: "IntegerPair subnet"
  "web \<equiv> {{(a,b). a = 8 }}"

  internet :: "IntegerPair subnet"
  "internet \<equiv> {{(a,b).  \<not> ( (a > 1 \<and> a < 4) \<or> (a > 6) \<and> (a < 11))   }}"

constdefs 
  Intranet_mail :: "(IntegerPair,'b) Rule"
  "Intranet_mail \<equiv> (allow_prots_from_to {smtp,imap} intranet mail)" 

  Intranet_Internet :: "(IntegerPair,'b) Rule"
  "Intranet_Internet \<equiv> allow_prots_from_to {http,https} intranet internet"
     
  Internet_web :: "(IntegerPair,'b) Rule"
  "Internet_web \<equiv> (allow_prots_from_to {http,https} internet web)"
          
  Internet_mail :: "(IntegerPair,'b) Rule"
  "Internet_mail \<equiv>  (allow_prot_from_to smtp internet dmz)"

constdefs 
  Intranet_mail_pp :: "(IntegerPair,'b) Rule"
  "Intranet_mail_pp \<equiv> (allow_prots_from_port_to {(smtp,21::port),(imap,24)} intranet mail)" 

  Intranet_Internet_pp :: "(IntegerPair,'b) Rule"
  "Intranet_Internet_pp \<equiv> allow_prots_from_port_to {(http,80::port),(https,90)} intranet internet"
     
  Internet_web_pp :: "(IntegerPair,'b) Rule"
  "Internet_web_pp \<equiv> (allow_prots_from_port_to {(http,80::port),(https,90)} internet web)"
          
  Internet_mail_pp :: "(IntegerPair,'b) Rule"
  "Internet_mail_pp \<equiv>  (allow_prot_from_port_to smtp internet (21::port) dmz)"

constdefs 
  Intranet_mail_port :: "(IntegerPair,'b) Rule"
  "Intranet_mail_port \<equiv> (allow_from_ports_to {21::port,14} intranet mail)" 

  Intranet_Internet_port :: "(IntegerPair,'b) Rule"
  "Intranet_Internet_port \<equiv> allow_from_ports_to {80::port,90} intranet internet"
     
  Internet_web_port :: "(IntegerPair,'b) Rule"
  "Internet_web_port \<equiv> (allow_from_ports_to {80::port,90} internet web)"
          
  Internet_mail_port :: "(IntegerPair,'b) Rule"
  "Internet_mail_port \<equiv>  (allow_all_from_port_to internet (21::port) dmz)"



constdefs 
  policyProtocol :: "(IntegerPair, DummyContent) Policy"
 "policyProtocol \<equiv> deny_all ++ 
                  Intranet_Internet ++ 
                  Intranet_mail ++  
                  Internet_mail ++ 
                  Internet_web"

constdefs 
  policyProtocolPort :: "(IntegerPair, DummyContent) Policy"
 "policyProtocolPort \<equiv> deny_all ++ 
                  Intranet_Internet_pp ++ 
                  Intranet_mail_pp ++  
                  Internet_mail_pp ++ 
                  Internet_web_pp"

constdefs 
  policyPort :: "(IntegerPair, DummyContent) Policy"
 "policyPort \<equiv> deny_all ++ 
                  Intranet_Internet_port ++ 
                  Intranet_mail_port ++  
                  Internet_mail_port ++ 
                  Internet_web_port"


text {* We only want to create test cases which are sent between the three main networks --- e.g. not between the mailserver and the dmz. Therefore, the constraint looks as follows. *}

constdefs
 not_in_same_net :: "(IntegerPair,DummyContent) packet \<Rightarrow> bool"
 "not_in_same_net x  \<equiv>  (src x \<sqsubset> internet \<longrightarrow> \<not> dest x \<sqsubset> internet) \<and> 
                        (src x \<sqsubset> intranet \<longrightarrow> dest x \<sqsubset> intranet) \<and> 
                        (src x \<sqsubset> dmz \<longrightarrow> dest x \<sqsubset> dmz)"

lemmas PolicyLemmas =  policyProtocol_def policyPort_def policyProtocolPort_def dmz_def internet_def intranet_def mail_def web_def 
                       Intranet_Internet_def Intranet_mail_def 
                       Internet_web_def Internet_mail_def



test_spec "not_in_same_net x \<longrightarrow> FUT (x:: (IntegerPair,DummyContent) packet) = policyProtocol x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas IntegerNetworkLemmas PortCombinators stateless_rules )
apply (gen_test_cases "FUT")
txt {* 662 test cases at this point, before unfolding the constraint or simplifiying (currently, there are even things like "\<not>6<7" in the assumptions\<dots>) *}
apply simp_all
apply (simp_all add: not_in_same_net_def src_def dest_def in_subnet_def PolicyLemmas)
apply(tactic {* TRYALL((arith_tac)) *})
store_test_thm "test"


gen_test_data "test"


thm test.test_data


end