ML{* add_path "../network_models/"; *}
ML{* add_path "../"; *}

theory Personal2Simplified
imports Main IntegerNetwork IntegerPairCombinators IPv4Combinators DatatypeTwoNetsNoPorts DatatypeTwoNetsCombinators PolicyCombinators Testing
begin


text{*
Same as Personal2, but with a simplified, tough semantically equivalent, policy. 
*}


section{* 1 Simple Integer, no ports *}

subsubsection {* 1.a Policy is complete *}

constdefs 
 PC :: "IntegerNetwork subnet"
"PC \<equiv> {{a. a = 3 \<or> a = 4}}"

 Internet :: "IntegerNetwork subnet"
"Internet \<equiv> {{a. a \<noteq> 3 \<and> a \<noteq> 4}}" 

constdefs
 Policy :: "(IntegerNetwork,DummyContent) Policy"
"Policy  \<equiv> deny_all ++ allow_prots_from_to PC Internet {http,https,smtp}"


lemmas PolicyLemmas = PC_def Internet_def Policy_def

lemma policy_complete: "\<not> (\<exists> x. Policy x = None)"
apply (simp add: PolicyLemmas IntegerNetworkLemmas stateless_rules)
done


constdefs not_in_same_net :: "(IntegerNetwork, 'a) packet \<Rightarrow> bool"
 "not_in_same_net x \<equiv> ((src x \<sqsubset> Internet) \<longrightarrow> (dest x \<sqsubset> PC)) \<and> ((src x \<sqsubset> PC) \<longrightarrow> (dest x \<sqsubset> Internet))"

test_spec "not_in_same_net x \<longrightarrow> FUT x = Policy x"
  apply (rule_tac x=x in spec, rule allI)
apply (rule impI)
   apply (simp only: split_tupled_all)
apply (simp add: PolicyLemmas IntegerNetworkLemmas stateless_rules dest_def)
apply (gen_test_cases "FUT")
apply (simp_all add: PolicyLemmas  IntegerNetworkLemmas not_in_same_net_def)
apply (simp_all add: src_def dest_def)
store_test_thm "test"

gen_test_data "test"

thm test.test_data (* 6 test cases. Possible PROBLEM: Depending on the outcome of the random phase, possibly NO test cases with denied protocols from the PC to the Internet. *)

thm test.test_thm



subsection{* Variant 1.b Policy underspecified *}

constdefs
 Policy2 :: "(IntegerNetwork,'a) Policy"
"Policy2  \<equiv> deny_all_from_to Internet PC ++ deny_all_from_to PC Internet ++ allow_prots_from_to  PC Internet {http,https,smtp}"


test_spec "not_in_same_net x \<longrightarrow> FUT x = Policy2 x"
  apply (rule_tac x=x in spec, rule allI) 
txt{* 
  This step is needed to introduce a meta-all-quantor which is
  necessary for the following split rule.
*}
apply (rule impI)
   apply (simp only: not_in_same_net_def split_tupled_all)
apply (simp add: Policy2_def stateless_rules src_def dest_def protocol_is PolicyLemmas in_subnet_def)
apply (gen_test_cases "FUT")
store_test_thm "test"

gen_test_data "test"

thm test.test_data  (* Five test cases - this time including a denied case from the PC --- this is because the policy talks explicitly about it *)


(*

text{* We see that the first gives us three test cases, while the latter only two. Is that because of the different specification of the policy, or because we unfold the constraint later? Let's try again the first possibility, but unfold it earlier this time. *}

*)


subsubsection {* 1.c Policy is complete, unfold not_in_same_net earlier.  *}


test_spec "not_in_same_net x \<longrightarrow> FUT x = Policy x"
  apply (rule_tac x=x in spec, rule allI)
apply (rule impI)
   apply (simp only: split_tupled_all)
apply (simp add: not_in_same_net_def Policy_def IntegerNetworkLemmas src_def dest_def in_subnet_def stateless_rules PC_def Internet_def)
apply (gen_test_cases "FUT")
store_test_thm "test"

gen_test_data "test"

thm test.test_data (* Same as 1a *)




(****************************************************************************************************************************************************)


text{* From now on we only do the fully specified policy *}

section {* 2 A network is a pair of Integers (resp. a range), first the adress, second the port *}
(*


constdefs 
 PC2 :: "IntegerPair subnet"
"PC2 \<equiv> {{(a,b) . a = 3}}"

 Internet2 :: "IntegerPair subnet"
"Internet2 \<equiv> {{(a,b). a \<noteq> 3}}" 

constdefs
 Policy22 :: "(IntegerPair,'a) Policy"
"Policy22  \<equiv> deny_all ++ 
            (allow_prots_from_port_to PC2 Internet2 {(http,80),(https,443),(smtp,21)})"

lemmas PolicyLemmas = PC2_def Internet2_def Policy22_def

lemma policy_complete: "\<not> (\<exists> x. Policy22 x = None)"
apply (simp add: PolicyLemmas IntegerNetworkLemmas IntegerPairCombinators stateless_rules)
done


constdefs not_in_same_net2 :: "(IntegerPair, 'a) packet \<Rightarrow> bool"
 "not_in_same_net2 x \<equiv> ((src x \<sqsubset> Internet2) \<longrightarrow> (dest x \<sqsubset> PC2)) \<and> ((src x \<sqsubset> PC2) \<longrightarrow> (dest x \<sqsubset> Internet2))"


ML{* TestGen.trace_test := false *}

test_spec "not_in_same_net2 x \<longrightarrow> FUT x = Policy22 x"
  apply (rule_tac x=x in spec, rule allI)
apply (rule impI)

   apply (simp only: split_tupled_all)
apply (simp add: PolicyLemmas IntegerPairCombinators not_in_same_net2_def stateless_rules)
apply (simp add: src_def dest_def in_subnet_def src_port_def dest_port_def)

apply (simp add: protocol_def)
apply (gen_test_cases "FUT") 

txt{* There are some very bad test cases --- obviously the simplifier does too much. The following two instruction rewrite the problematic cases into a format which makes them usable again for testing. *}


apply(tactic {* TRYALL((eresolve_tac  [(thm "make_pos_goal")])) *})
apply(tactic {* TRYALL((thin_tac  "FUT ?X \<noteq> Some deny")) *})
apply simp_all

txt {* 

RESOLVED: The following is no longer true:

This is a very bad result. We get 65 test cases, but many of them are useless! (i.e. \<lbrakk>foo\<rbrakk> \<Longrightarrow> False )  
       This situation also arises, if we do not include the not_in_same_network constraint. 
       Interestingly, it even arises if we remove the ports from the policy! However, if we remove the constraint AND the ports AND make the policy smaller, it does not.
       Before the gen_test_cases phase, the lemma looks very similar to the one of scenario 5 (where the problem does not occur) - with the exception that in the latter, the simplifier can derive that not ONE must mean TWO, where as here we only know "not 3", and therefore cannot instantiate the corresponding variable already.

  *}

store_test_thm "test"

gen_test_data "test"

thm test.test_thm

thm test.test_data  (* 65 test data, only three of them are accept (makes sense, as the policy only allows three kinds of packets through the firewall. *)

*)

section {*  3 A network is an tuple of four integers, with ports *}


constdefs 
 PC3 :: "ipv4 subnet"
"PC3 \<equiv> {{((a,b,c,d),e) . a = 3 \<and> b = 2 \<and> c = 1 \<and> d = 1}}"

 Internet3 :: "ipv4 subnet"
"Internet3 \<equiv> {{((a,b,c,d),e). \<not> ( a = 3 \<and> b = 2 \<and> c = 1 \<and> d = 1) }}" 


text{* Note that it is EXTREMELY significant if we model the policy the first (old) or the second way *}


constdefs
 Policy3 :: "(ipv4,'a) Policy"
"Policy3  \<equiv> deny_all ++ IPv4Combinators.allow_prots_from_port_to PC3 Internet3 {(http,80),(https,443),(smtp,21)}"

lemmas PolicyLemmas = PC3_def Internet3_def Policy3_def

lemma policy_complete: "\<not> (\<exists> x. Policy3 x = None)"
apply (simp add: PolicyLemmas IPv4Combinators stateless_rules)
done


constdefs not_in_same_net3 :: "(ipv4, 'a) packet \<Rightarrow> bool"
 "not_in_same_net3 x \<equiv> ((src x \<sqsubset> Internet3) \<longrightarrow> (dest x \<sqsubset> PC3)) \<and> ((src x \<sqsubset> PC3) \<longrightarrow> (dest x \<sqsubset> Internet3))"

test_spec "not_in_same_net3 x \<longrightarrow> FUT x = Policy3 x"
  apply (rule_tac x=x in spec, rule allI)
apply (rule impI)
   apply (simp only: split_tupled_all)
apply (simp add: PolicyLemmas IPv4Combinators IPv4Lemmas stateless_rules)
apply (gen_test_cases "FUT")
apply (simp_all add: PolicyLemmas IPv4Combinators IPv4Lemmas not_in_same_net3_def)


store_test_thm "test"

gen_test_data "test"

thm test.test_data (* 25 test cases *)
thm test.test_thm
text{*
Due to the more detailed representation of a network address, we get a significant blow-up in the number of test cases. Note that here we still have the very simple policy where we do NOT talk about specific protocols or port numbers. *}



section {*   4 The networks are modelled as a datatype, no ports. *}


constdefs 
 PC4 :: "DatatypeTwoNetsNoPorts subnet"
"PC4 \<equiv> {{ONE}}"

 Internet4 :: "DatatypeTwoNetsNoPorts subnet"
"Internet4 \<equiv> {{TWO}}" 

constdefs
 Policy4 :: "(DatatypeTwoNetsNoPorts,'a) Policy"
"Policy4  \<equiv> deny_all ++ allow_prots_from_to PC4 Internet4 {http,https,smtp}"

lemmas PolicyLemmas = PC4_def Internet4_def Policy4_def

lemma policy_complete: "\<not> (\<exists> x. Policy4 x = None)"
apply (simp add: PolicyLemmas stateless_rules)
done


constdefs not_in_same_net4 :: "(DatatypeTwoNetsNoPorts, 'a) packet \<Rightarrow> bool"
 "not_in_same_net4 x \<equiv> ((src x \<sqsubset> Internet4) \<longrightarrow> (dest x \<sqsubset> PC4)) \<and> ((src x \<sqsubset> PC4) \<longrightarrow> (dest x \<sqsubset> Internet4))"


text {* When we model the networks as datatype, the simplifier has some problems in noticing that each network must be distinct. One possible solution is to use the following lemma when needed (not in this example). 

However, anyway, the not_in_same_net definition must only be unfolded AFTER the test case generation phase!!! *}

lemma foo: "({(a, b). a = c} \<noteq>  {(a, b). a = d}) ==> (c \<noteq>  d )"
apply auto
done

test_spec "not_in_same_net4 x \<longrightarrow> FUT x = Policy4 x"
  apply (rule_tac x=x in spec, rule allI)
apply (rule impI)
   apply (simp only: split_tupled_all)
apply (simp_all add: PolicyLemmas DatatypeTwoNetsNoPortsLemmas stateless_rules)
apply (simp add: src_def dest_def in_subnet_def)

apply (gen_test_cases "FUT")
apply (simp_all add: PolicyLemmas DatatypeTwoNetsNoPortsLemmas not_in_same_net4_def)
apply (simp_all add: src_def dest_def in_subnet_def)
store_test_thm "test"

gen_test_data "test"

thm test.test_data (* 6 test cases - here we have the same problem as in 1.a), there might be no denied packets from PC to Internet. *)


section {* 5 The networks are modelled as a datatype, with ports *}


constdefs 
 PC5 :: "DatatypeTwoNets subnet"
"PC5 \<equiv> {{(a,b). a = one}}"

 Internet5 :: "DatatypeTwoNets subnet"
"Internet5 \<equiv> {{(a,b). a = two}}" 

constdefs
 Policy5 :: "(DatatypeTwoNets,'a) Policy"
"Policy5  \<equiv> deny_all ++ DatatypeTwoNetsCombinators.allow_prots_from_port_to PC5 Internet5 {(http,80),(https,443),(smtp,21)}"

lemmas PolicyLemmas = PC5_def Internet5_def Policy5_def

lemma policy_complete: "\<not> (\<exists> x. Policy5 x = None)"
apply (simp add: PolicyLemmas stateless_rules DatatypeTwoNetsCombinators)
done


constdefs not_in_same_net5 :: "(DatatypeTwoNets, 'a) packet \<Rightarrow> bool"
 "not_in_same_net5 x \<equiv> ((src x \<sqsubset> Internet5) \<longrightarrow> (dest x \<sqsubset> PC5)) \<and> ((src x \<sqsubset> PC5) \<longrightarrow> (dest x \<sqsubset> Internet5))"


test_spec "not_in_same_net5 x \<longrightarrow> FUT x = Policy5 x"
  apply (rule_tac x=x in spec, rule allI)
apply (rule impI)
   apply (simp only: split_tupled_all)
apply (simp_all add: PolicyLemmas DatatypeTwoNetsLemmas DatatypeTwoNetsCombinators stateless_rules)
apply (gen_test_cases "FUT")
apply (simp_all add: PolicyLemmas DatatypeTwoNetsLemmas not_in_same_net5_def)
store_test_thm "test"

gen_test_data "test"

thm test.test_data (* This already gives a very large amounts of test data: 120. Here we can cleary see the large blow-up induced by including the protocols and the ports into the policy.

As in 2, three of the test cases are accepting ones.

Interestingly, here the wrong test cases of 2 have never appeared.
 *)

thm test.test_thm


section{* 6 Range of Integers, no ports *}

subsubsection {* 1.a Policy is complete *}

constdefs 
 PC6 :: "IntegerNetwork subnet"
"PC6 \<equiv> {{a. a > 2 \<and> a < 6}}"

 Internet6 :: "IntegerNetwork subnet"
"Internet6 \<equiv> {{a. a < 3 \<or> a > 5}}" 

constdefs
 Policy6 :: "(IntegerNetwork,'a) Policy"
"Policy6  \<equiv> deny_all ++ (allow_prots_from_to PC6 Internet6 {http,https,smtp})"


lemmas PolicyLemmas = PC6_def Internet6_def Policy6_def

lemma policy_complete: "\<not> (\<exists> x. Policy6 x = None)"
apply (simp add: PolicyLemmas IntegerNetworkLemmas stateless_rules)
done


constdefs not_in_same_net6 :: "(IntegerNetwork, 'a) packet \<Rightarrow> bool"
 "not_in_same_net6 x \<equiv> ((src x \<sqsubset> Internet6) \<longrightarrow> (dest x \<sqsubset> PC6)) \<and> ((src x \<sqsubset> PC6) \<longrightarrow> (dest x \<sqsubset> Internet6))"

test_spec "not_in_same_net6 x \<longrightarrow> FUT x = Policy6 x"
  apply (rule_tac x=x in spec, rule allI)
apply (rule impI)
   apply (simp only: split_tupled_all)
apply (simp add: PolicyLemmas IntegerNetworkLemmas stateless_rules dest_def)
apply (simp add: src_def dest_def)
apply (gen_test_cases "FUT")
apply (simp_all add: PolicyLemmas  IntegerNetworkLemmas not_in_same_net6_def)
apply (simp_all add: src_def dest_def)
store_test_thm "test"

gen_test_data "test"

thm test.test_data (* 10 test cases, six of which are accepting. 
                      Here, it can happen that we get a RSF, this happens if the random generator guesses the wrong numbers too many times.
 *)



text {* Some observations (to be continued):
 
 - There is some blow-up as soon as we add protocols to the policy.

 - The blow-up gets very large as soons as we additionally add the ports. Now we have three factors instead of only one.

 - Scenario 4, where adresses are modelled as four-tuples, already needs an extremly considerable amount of time.

*}




end
 


 
