theory "ethz-01"
imports IntegerPairNetwork PortCombinators "../../../src/FWTesting"
begin



(* The subnetworks *)

(* 
 Mapping to the subnetworks

 129.132.240.0/26     \<Longrightarrow>  1
 129.132.176.160/27   \<Longrightarrow>  2
 FW-LINK              \<Longrightarrow>  3
 129.132.254.33/32    \<Longrightarrow>  4
 224.0.0.0/4          \<Longrightarrow>  5
 SIP-Server           \<Longrightarrow>  6
 time.ethz.ch         \<Longrightarrow>  7
 DHCP                 \<Longrightarrow>  8
 129.132.105.4/32     \<Longrightarrow>  9
 129.132.105.10/32    \<Longrightarrow>  10

 ETHZ-Intern          \<Longrightarrow>  11
 ETHZ-Private         \<Longrightarrow>  12
 VPN-VPZ              \<Longrightarrow>  13


*)

constdefs
 FWLink :: "IntegerPair subnet"
"FWLink \<equiv> {{(a,b). a = 3}}"

 any :: "IntegerPair subnet"
"any \<equiv> {{(a,b). a > 13}}"

 i26:: "IntegerPair subnet"
"i26 \<equiv> {{(a,b). a = 1 }}"

 i27:: "IntegerPair subnet"
"i27 \<equiv> {{(a,b). a = 2 }}"

 i32:: "IntegerPair subnet"
"i32 \<equiv> {{(a,b). a =  4}}"

 i4:: "IntegerPair subnet"
"i4 \<equiv> {{(a,b). a = 5 }}"

 sip:: "IntegerPair subnet"
"sip \<equiv> {{(a,b). a = 6 }}"

 time:: "IntegerPair subnet"
"time \<equiv> {{(a,b). a = 7}}"

 dhcp:: "IntegerPair subnet"
"dhcp \<equiv> {{(a,b). a =  8}}"
 
 i4_32:: "IntegerPair subnet"
"i4_32 \<equiv> {{(a,b). a =  9}}"

 i10_32:: "IntegerPair subnet"
"i10_32 \<equiv> {{(a,b). a = 10 }}"

 eth_intern:: "IntegerPair subnet"
"eth_intern \<equiv> {{(a,b). a = 11 }}"
 
 eth_private:: "IntegerPair subnet"
"eth_private \<equiv> {{(a,b). a = 12 }}"

 vpn_vpz :: "IntegerPair subnet"
"vpn_vpz \<equiv> {{(a,b). a = 13}}"


constdefs
(* Mandatory: Global *)

 MG1 :: "(IntegerPair,DummyContent) Policy"
"MG1 \<equiv> allow_prot_from_to icmp i26 any"

 MG2 :: "(IntegerPair,DummyContent) Policy"
"MG2 \<equiv> allow_prots_from_to {icmp,tcp,udp} i27 any"

 MG3 :: "(IntegerPair,DummyContent) Policy"
"MG3 \<equiv> allow_prot_from_to icmp any FWLink"

 MG4 :: "(IntegerPair,DummyContent) Policy"
"MG4 \<equiv> allow_prot_from_to bgp FWLink FWLink
"
 MG5 :: "(IntegerPair,DummyContent) Policy"
"MG5 \<equiv> allow_prot_from_to pim i32 any"

 MG6 :: "(IntegerPair,DummyContent) Policy"
"MG6 \<equiv> allow_prot_from_to igmp i32 i4 "
 
 MG7 :: "(IntegerPair,DummyContent) Policy"
"MG7 \<equiv> allow_prots_from_to {igmp,IP} FWLink i4"

 MG8 :: "(IntegerPair,DummyContent) Policy"
"MG8 \<equiv> allow_prots_from_to {igmp,IP} FWLink i4"

 MG9 :: "(IntegerPair,DummyContent) Policy"
"MG9 \<equiv> allow_prot_from_to pim any i32 "

 MG10 :: "(IntegerPair,DummyContent) Policy"
"MG10 \<equiv> allow_prot_from_to pim i32 any"

 MG11 :: "(IntegerPair,DummyContent) Policy"
"MG11 \<equiv> allow_prot_from_to icmp FWLink any"

 MG12 :: "(IntegerPair,DummyContent) Policy"
"MG12 \<equiv> allow_prot_from_to bgp FWLink FWLink"

 MG13 :: "(IntegerPair,DummyContent) Policy"
"MG13 \<equiv> allow_prots_from_to {udp, tcp} sip any "

(* Default Global *)

 DG1 :: "(IntegerPair,DummyContent) Policy"
"DG1 \<equiv> allow_prot_from_to ntp time any "

 DG2:: "(IntegerPair,DummyContent) Policy"
"DG2 \<equiv> allow_prot_from_to icmp dhcp any "

 DG3:: "(IntegerPair,DummyContent) Policy"
"DG3 \<equiv> deny_prot_from_to IP any any "

 


constdefs 
 MandatoryGlobalPolicy :: "(IntegerPair,DummyContent) Policy"
"MandatoryGlobalPolicy \<equiv> MG13 ++ MG12 ++  MG11 ++  MG10 ++  MG9 ++  MG8 ++  MG7 ++  MG6 ++ MG5 ++ MG4 ++ MG3 ++ MG2 ++ MG1"   
      
 MandatoryGlobalInsidePolicy :: "(IntegerPair,DummyContent) Policy"
"MandatoryGlobalInsidePolicy \<equiv> MG12 ++  MG11 ++  MG10 ++  MG9 ++  MG8"
         
 MandatoryGlobalOutsidePolicy :: "(IntegerPair,DummyContent) Policy"
"MandatoryGlobalOutsidePolicy \<equiv>   MG13 ++ MG7 ++ MG6 ++ MG5 ++ MG4 ++ MG3 ++ MG2 ++ MG1"       

 DefaultGlobalPolicy :: "(IntegerPair,DummyContent) Policy"
"DefaultGlobalPolicy \<equiv> DG3 ++ DG2 ++ DG1"  

 lemmas PolicyLemmas =  
 FWLink_def
 any_def
 i26_def
 i27_def
 i32_def
 i4_def
 sip_def
 time_def
 dhcp_def
 i4_32_def
 i10_32_def
 eth_intern_def
 eth_private_def
 vpn_vpz_def
 MG1_def MG2_def MG3_def MG4_def MG5_def MG6_def MG7_def MG8_def MG9_def MG10_def MG11_def MG12_def MG13_def
 DG1_def DG2_def DG3_def
 MandatoryGlobalPolicy_def MandatoryGlobalOutsidePolicy_def MandatoryGlobalInsidePolicy_def DefaultGlobalPolicy_def

constdefs 
 not_in_same_net :: "(IntegerPair,DummyContent) packet \<Rightarrow> bool"
"not_in_same_net x \<equiv>  ((src x \<sqsubset> i26 ) \<longrightarrow> ( \<not> (dest x \<sqsubset> i26 ))) \<and>  
                      ((src x \<sqsubset> i27) \<longrightarrow> ( \<not> (dest x \<sqsubset> i27))) \<and>
                      ((src x \<sqsubset> i32 ) \<longrightarrow> ( \<not> (dest x \<sqsubset> i32))) \<and>
                      ((src x \<sqsubset>  i4) \<longrightarrow> ( \<not> (dest x \<sqsubset> i4))) \<and>
                      ((src x \<sqsubset>  sip) \<longrightarrow> ( \<not> (dest x \<sqsubset> sip))) \<and>
                      ((src x \<sqsubset>  time) \<longrightarrow> ( \<not> (dest x \<sqsubset> time))) \<and>
                      ((src x \<sqsubset>  dhcp) \<longrightarrow> ( \<not> (dest x \<sqsubset> dhcp))) \<and>
                      ((src x \<sqsubset>  i4_32) \<longrightarrow> ( \<not> (dest x \<sqsubset> i4_32))) \<and>
                      ((src x \<sqsubset>  i10_32) \<longrightarrow> ( \<not> (dest x \<sqsubset> i10_32))) \<and>
                      ((src x \<sqsubset>  eth_intern) \<longrightarrow> ( \<not> (dest x \<sqsubset> eth_intern))) \<and>
                      ((src x \<sqsubset>  vpn_vpz) \<longrightarrow> ( \<not> (dest x \<sqsubset> vpn_vpz))) \<and>
                      ((src x \<sqsubset>  eth_private) \<longrightarrow> ( \<not> (dest x \<sqsubset> eth_private)))"
                   
(* Commented out as others import it - would take too long
*)

test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ DefaultGlobalPolicy) x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas IntegerNetworkLemmas PortCombinators stateless_rules)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas IntegerNetworkLemmas) 
store_test_thm "DG"


gen_test_data "DG"
thm DG.test_data

test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalInsidePolicy) x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas  IntegerNetworkLemmas PortCombinators stateless_rules)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas IntegerNetworkLemmas) 
store_test_thm "MGI"


gen_test_data "MGI"
thm MGI.test_data

test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalOutsidePolicy) x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas  IntegerNetworkLemmas PortCombinators stateless_rules)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas IntegerNetworkLemmas) 
store_test_thm "MGO"


gen_test_data "MGO"
thm MGO.test_data

test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalPolicy) x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas  IntegerNetworkLemmas PortCombinators stateless_rules)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas IntegerNetworkLemmas) 
store_test_thm "MG"


gen_test_data "MG"
thm MG.test_data


test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ DefaultGlobalPolicy ++ MandatoryGLobalPolicy) x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas IntegerNetworkLemmas PortCombinators stateless_rules)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def in_subnet_def src_def dest_def PolicyLemmas IntegerNetworkLemmas) 
store_test_thm "all"


gen_test_data "all"
thm all.test_data


end
