theory "ethz-01Datatype"
imports PolicyCombinators "../../../src/FWTesting"
begin



(* The subnetworks *)

datatype networks =  FWLInk |
 Any |
 I26|
 I27|
 I32|
 I4|
 Sip|
 Time|
 Dhcp|
 I4_32|
 I10_32|
 Eth_intern|
 Eth_private|
 Vpn_vpz
  
 
instance networks :: net ..


constdefs
 FWLink :: "networks subnet"
"FWLink \<equiv> {{a. a = FWLInk}}"

 any :: "networks subnet"
"any \<equiv> {{a. a = Any}}"

 i26:: "networks subnet"
"i26 \<equiv> {{a. a = I26}}"

 i27:: "networks subnet"
"i27 \<equiv> {{a. a = I27 }}"

 i32:: "networks subnet"
"i32 \<equiv> {{a. a =  I32}}"

 i4:: "networks subnet"
"i4 \<equiv> {{a. a = I4 }}"

 sip:: "networks subnet"
"sip \<equiv> {{a. a = Sip }}"

 time:: "networks subnet"
"time \<equiv> {{a. a = Time}}"

 dhcp:: "networks subnet"
"dhcp \<equiv> {{a. a =  Dhcp}}"
 
 i4_32:: "networks subnet"
"i4_32 \<equiv> {{a. a =  I4_32}}"

 i10_32:: "networks subnet"
"i10_32 \<equiv> {{a. a = I10_32}}"

 eth_intern:: "networks subnet"
"eth_intern \<equiv> {{a. a = Eth_intern }}"
 
 eth_private:: "networks subnet"
"eth_private \<equiv> {{a. a = Eth_private}}"

 vpn_vpz :: "networks subnet"
"vpn_vpz \<equiv> {{a. a = Vpn_vpz}}"


constdefs
(* Mandatory: Global *)

 MG1 :: "(networks,DummyContent) Policy"
"MG1 \<equiv> allow_prot_from_to icmp i26 any"

 MG2 :: "(networks,DummyContent) Policy"
"MG2 \<equiv> allow_prots_from_to {icmp,tcp,udp} i27 any"

 MG3 :: "(networks,DummyContent) Policy"
"MG3 \<equiv> allow_prot_from_to icmp any FWLink"

 MG4 :: "(networks,DummyContent) Policy"
"MG4 \<equiv> allow_prot_from_to bgp FWLink FWLink
"
 MG5 :: "(networks,DummyContent) Policy"
"MG5 \<equiv> allow_prot_from_to pim i32 any"

 MG6 :: "(networks,DummyContent) Policy"
"MG6 \<equiv> allow_prot_from_to igmp i32 i4 "
 
 MG7 :: "(networks,DummyContent) Policy"
"MG7 \<equiv> allow_prots_from_to {igmp,IP} FWLink i4"

 MG8 :: "(networks,DummyContent) Policy"
"MG8 \<equiv> allow_prots_from_to {igmp,IP} FWLink i4"

 MG9 :: "(networks,DummyContent) Policy"
"MG9 \<equiv> allow_prot_from_to pim any i32 "

 MG10 :: "(networks,DummyContent) Policy"
"MG10 \<equiv> allow_prot_from_to pim i32 any"

 MG11 :: "(networks,DummyContent) Policy"
"MG11 \<equiv> allow_prot_from_to icmp FWLink any"

 MG12 :: "(networks,DummyContent) Policy"
"MG12 \<equiv> allow_prot_from_to bgp FWLink FWLink"

 MG13 :: "(networks,DummyContent) Policy"
"MG13 \<equiv> allow_prots_from_to {udp, tcp} sip any "

(* Default Global *)

 DG1 :: "(networks,DummyContent) Policy"
"DG1 \<equiv> allow_prot_from_to ntp time any "

 DG2:: "(networks,DummyContent) Policy"
"DG2 \<equiv> allow_prot_from_to icmp dhcp any "

 DG3:: "(networks,DummyContent) Policy"
"DG3 \<equiv> deny_prot_from_to IP any any "

 


constdefs 
 MandatoryGlobalPolicy :: "(networks,DummyContent) Policy"
"MandatoryGlobalPolicy \<equiv> MG13 ++ MG12 ++  MG11 ++  MG10 ++  MG9 ++  MG8 ++  MG7 ++  MG6 ++ MG5 ++ MG4 ++ MG3 ++ MG2 ++ MG1"   
      
 MandatoryGlobalInsidePolicy :: "(networks,DummyContent) Policy"
"MandatoryGlobalInsidePolicy \<equiv> MG12 ++  MG11 ++  MG10 ++  MG9 ++  MG8"
         
 MandatoryGlobalOutsidePolicy :: "(networks,DummyContent) Policy"
"MandatoryGlobalOutsidePolicy \<equiv>   MG13 ++ MG7 ++ MG6 ++ MG5 ++ MG4 ++ MG3 ++ MG2 ++ MG1"       

 DefaultGlobalPolicy :: "(networks,DummyContent) Policy"
"DefaultGlobalPolicy \<equiv> DG3 ++ DG2 ++ DG1"  

 lemmas PolicyLemmas =  
 FWLink_def
 any_def
 i26_def
 i27_def
 i32_def
 i4_def
 sip_def
 time_def
 dhcp_def
 i4_32_def
 i10_32_def
 eth_intern_def
 eth_private_def
 vpn_vpz_def
 MG1_def MG2_def MG3_def MG4_def MG5_def MG6_def MG7_def MG8_def MG9_def MG10_def MG11_def MG12_def MG13_def
 DG1_def DG2_def DG3_def
 MandatoryGlobalPolicy_def MandatoryGlobalOutsidePolicy_def MandatoryGlobalInsidePolicy_def DefaultGlobalPolicy_def
 src_def dest_def protocol_def in_subnet_def

constdefs 
 not_in_same_net :: "(networks,DummyContent) packet \<Rightarrow> bool"
"not_in_same_net x \<equiv>  ((src x \<noteq> dest x))"                   

test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ DefaultGlobalPolicy) x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas stateless_rules)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas) 
store_test_thm "DG"


gen_test_data "DG"
thm DG.test_data

test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalInsidePolicy) x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas stateless_rules)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas) 
store_test_thm "MGI"


gen_test_data "MGI"
thm MGI.test_data

test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalOutsidePolicy) x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas stateless_rules)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas) 
store_test_thm "MGO"


gen_test_data "MGO"
thm MGO.test_data

test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ MandatoryGlobalPolicy) x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas PortCombinators stateless_rules)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas) 
store_test_thm "MG"


gen_test_data "MG"
thm MG.test_data


test_spec "not_in_same_net x \<longrightarrow> FUT x =  (deny_all ++ DefaultGlobalPolicy ++ MandatoryGLobalPolicy) x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas  stateless_rules)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas) 
store_test_thm "all"


gen_test_data "all"
thm all.test_data


end