theory fourSubnets
imports Main PolicyCombinators "../../src/FWTesting" 
begin


datatype Networks = One | Two | Three | Four

instance Networks::net .. 

constdefs
  one::"Networks subnet"
  "one \<equiv> {{One}}"
  two :: "Networks subnet"
  "two \<equiv> {{Two}}"
  three :: "Networks subnet"
  "three \<equiv> {{Three}}"
  four :: "Networks subnet"
  "four \<equiv> {{Four}}"
(*

constdefs 
  NaivePolicy :: "(Networks,'b) Rule"
 "NaivePolicy \<equiv> deny_all ++
                allow_prots_from_to {http,https} one two ++
                deny_all_from_to two one ++ 
                allow_prot_from_to ftp two one ++ 
                allow_prots_from_to {imap, voip} three one ++
                deny_prots_from_to {IP,icmp} four one ++
                allow_prots_from_to {smtp,imap,http,https} three two ++ 
                allow_prots_from_to {imap,https} four two ++ 
                deny_prot_from_to voip one two ++
                allow_prots_from_to {ftp,http,https} two four ++
                allow_prot_from_to imap two four ++ 
                allow_prot_from_to smtp three four ++ 
                deny_prot_from_to https three two ++
                allow_prots_from_to {icmp,ntp} three one ++
                allow_prot_from_to ntp three four ++
                allow_prot_from_to smtp one four ++
                deny_prots_from_to {http,imap} four two ++
                allow_prot_from_to https four two ++
                allow_prots_from_to {smtp,imap} two one"

*)


constdefs 
  NaivePolicy :: "(Networks,'b) Rule"
 "NaivePolicy \<equiv> deny_all ++
                deny_all_from_to two one ++ 
                allow_prot_from_to ftp two one ++ 
                deny_prot_from_to voip one two ++
                allow_prot_from_to imap two four ++ 
                allow_prot_from_to smtp three four ++ 
                deny_prot_from_to https three two ++
                allow_prot_from_to ntp three four ++
                allow_prot_from_to smtp one two"(*
                allow_prot_from_to https two four"
*)

constdefs
  SimplifiedPolicy :: "(Networks,'b) Rule"
 "SimplifiedPolicy \<equiv> deny_all ++
                     allow_prot_from_to ftp two one ++
                     allow_prots_from_to {voip,smtp} one two++
                     allow_prots_from_to {imap,https} two four ++
                     allow_prots_from_to {smtp,ntp} three four"
                    


text {* We only want to create test cases which are sent between the three main networks --- e.g. not between the mailserver and the dmz. Therefore, the constraint looks as follows. *}

constdefs
 not_in_same_net :: "(Networks,DummyContent) packet \<Rightarrow> bool"
 "not_in_same_net x  \<equiv>  (src x  \<noteq> dest x)"

constdefs
 not_in_same_net1 :: "(Networks,DummyContent) packet \<Rightarrow> bool"
 "not_in_same_net1 x  \<equiv>  (src x \<sqsubset> one \<longrightarrow> \<not> dest x \<sqsubset> one) \<and> 
                        (src x \<sqsubset> two \<longrightarrow> \<not> dest x \<sqsubset> two) \<and> 
                        (src x \<sqsubset> three \<longrightarrow> \<not> dest x \<sqsubset> three) \<and> 
                        (src x \<sqsubset> four \<longrightarrow> \<not> dest x \<sqsubset> four)"


lemmas PolicyLemmas =  NaivePolicy_def SimplifiedPolicy_def one_def two_def three_def four_def 
                   

test_spec "not_in_same_net x \<longrightarrow> FUT (x:: (Networks,DummyContent) packet) = NaivePolicy x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas src_def dest_def in_subnet_def protocol_def stateless_rules )
apply (gen_test_cases "FUT")
apply simp_all
apply (simp_all add: not_in_same_net_def src_def dest_def in_subnet_def PolicyLemmas)
store_test_thm "Simplified"

gen_test_data "Simplified"



test_spec "not_in_same_net x \<longrightarrow> FUT (x:: (Networks,DummyContent) packet) = NaivePolicy x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas src_def dest_def in_subnet_def protocol_def stateless_rules )
apply (gen_test_cases "FUT")
apply simp_all
apply (simp_all add: not_in_same_net_def src_def dest_def in_subnet_def PolicyLemmas)
store_test_thm "Naive"

gen_test_data "Naive"


end
