theory PersonalFirewallDatatypeWithoutPorts
imports PolicyCombinators  "../../src/FWTesting"

begin


text{*

The most basic firewall scenario; there is a personal PC on one side and the Internet on the other. 
There are two policies: the first one allows all traffic from the PC to the Internet and denies
all coming into the PC. The second policy only allows specific ports from the PC. This scenario
comes in three variants: the first one specifies the allowed protocols directly, the second together 
with their respective port numbers, the third one only with the port numbers.

*}



text{*
Definitions of the subnets 
*}

datatype DatatypeTwoNets = one | two

instance DatatypeTwoNets :: net ..


constdefs
 
 PC :: "(DatatypeTwoNets subnet)"
"PC \<equiv> {{one}}"

 Internet :: "DatatypeTwoNets subnet"
"Internet \<equiv> {{two}}"


text{*
Definition of the testing constraint
*}

constdefs
 not_in_same_net :: "(DatatypeTwoNets,'b) packet \<Rightarrow> bool"
"not_in_same_net x \<equiv> (src x \<sqsubset> PC \<longrightarrow> dest x \<sqsubset> Internet) \<and> (src x \<sqsubset> Internet \<longrightarrow> dest x \<sqsubset> PC)" 

text {*
Definitions of the policies 
*}

constdefs
 strictPolicy :: "(DatatypeTwoNets,'b) Policy"
"strictPolicy \<equiv> deny_all ++ allow_all_from_to PC Internet"

 ProtocolPolicy :: "(DatatypeTwoNets,'b) Policy"
"ProtocolPolicy \<equiv> deny_all ++ allow_prots_from_to {http,ftp,imap} PC Internet"


 ProtocolPolicyBig :: "(DatatypeTwoNets,'b) Policy"
"ProtocolPolicyBig \<equiv> deny_all ++ allow_prot_from_to http PC Internet ++ allow_prot_from_to ftp PC Internet++ allow_prot_from_to imap PC Internet"

lemmas policyLemmas = strictPolicy_def ProtocolPolicy_def  PC_def Internet_def ProtocolPolicyBig_def src_def dest_def in_subnet_def protocol_def


text{*

Testing the three policies

*}


test_spec "not_in_same_net x \<longrightarrow> FUT x = strictPolicy x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules )
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas )
store_test_thm "strict"

testgen_params[iterations=25]

gen_test_data "strict"


test_spec "not_in_same_net x \<longrightarrow> FUT x = ProtocolPolicy x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules )
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas )
store_test_thm "protocol"

gen_test_data "protocol"


(* BIG *)


test_spec "not_in_same_net x \<longrightarrow> FUT x = ProtocolPolicyBig x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas stateless_rules)
store_test_thm "protocolBig"

gen_test_data "protocolBig"


end