
theory PersonalFirewallIntWithoutPorts
imports "../../src/PacketFilter/IntegerNetwork" "../../src/PacketFilter/PortCombinators" "../../src/FWTesting"
begin


text{*

The most basic firewall scenario; there is a personal PC on one side and the Internet on the other. 
There are two policies: the first one allows all traffic from the PC to the Internet and denies
all coming into the PC. The second policy only allows specific ports from the PC. This scenario
comes in three variants: the first one specifies the allowed protocols directly, the second together 
with their respective port numbers, the third one only with the port numbers.

*}



text{*
Definitions of the subnets 
*}

constdefs
 
 PC :: "(IntegerNetwork subnet)"
"PC \<equiv> {{3}}"

 Internet :: "IntegerNetwork subnet"
"Internet \<equiv> {{a. \<not> (a = 3)}}"



text{*
Definition of the testing constraint
*}

constdefs
 not_in_same_net :: "(IntegerNetwork,'b) packet \<Rightarrow> bool"
"not_in_same_net x \<equiv> (src x \<sqsubset> PC \<longrightarrow> dest x \<sqsubset> Internet) \<and> (src x \<sqsubset> Internet \<longrightarrow> dest x \<sqsubset> PC)" 

text {*
Definitions of the policies 
*}

constdefs
 strictPolicy :: "(IntegerNetwork,'b) Policy"
"strictPolicy \<equiv> deny_all ++ allow_all_from_to PC Internet"

 ProtocolPolicy :: "(IntegerNetwork,'b) Policy"
"ProtocolPolicy \<equiv> deny_all ++ allow_prots_from_to {http,ftp,imap} PC Internet"

 ProtocolPolicyBig :: "(IntegerNetwork,'b) Policy"
"ProtocolPolicyBig \<equiv> deny_all ++ allow_prot_from_to http PC Internet ++ allow_prot_from_to ftp PC Internet++ allow_prot_from_to imap PC Internet"

 

lemmas policyLemmas = strictPolicy_def ProtocolPolicy_def PC_def Internet_def ProtocolPolicyBig_def

text{*

Testing the three policies

*}


test_spec "not_in_same_net x \<longrightarrow> FUT x = strictPolicy x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules IntegerLemmas src_def dest_def in_subnet_def)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas IntegerLemmas src_def dest_def in_subnet_def)
store_test_thm "strict"

gen_test_data "strict"


test_spec "not_in_same_net x \<longrightarrow> FUT x = ProtocolPolicy x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules IntegerLemmas src_def dest_def in_subnet_def)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas IntegerLemmas src_def dest_def in_subnet_def)
store_test_thm "protocol"

gen_test_data "strict"

test_spec "not_in_same_net x \<longrightarrow> FUT x = ProtocolPolicyBig x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules IntegerLemmas src_def dest_def in_subnet_def)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas IntegerLemmas src_def dest_def in_subnet_def)
store_test_thm "protocolBig"

gen_test_data "protocolBig"





end