
theory PersonalFirewallIpv4
imports "../../src/PacketFilter/IPv4" "../../src/PacketFilter/PortCombinators" "../../src/FWTesting"
begin


text{*

The most basic firewall scenario; there is a personal PC on one side and the Internet on the other. 
There are two policies: the first one allows all traffic from the PC to the Internet and denies
all coming into the PC. The second policy only allows specific ports from the PC. This scenario
comes in three variants: the first one specifies the allowed protocols directly, the second together 
with their respective port numbers, the third one only with the port numbers.

*}



text{*
Definitions of the subnets 
*}

constdefs
 
 PC :: "(ipv4 subnet)"
"PC \<equiv> {{((a,b,c,d),e). a = 1 \<and> b = 3 \<and> c = 5 \<and> d = 2}}"

 Internet :: "ipv4 subnet"
"Internet \<equiv> {{((a,b,c,d),e). \<not> (a = 1 \<and> b = 3 \<and> c = 5 \<and> d = 2)}}"


text{*
Definition of the testing constraint
*}

constdefs
 not_in_same_net :: "(ipv4,'b) packet \<Rightarrow> bool"
"not_in_same_net x \<equiv> (src x \<sqsubset> PC \<longrightarrow> dest x \<sqsubset> Internet) \<and> (src x \<sqsubset> Internet \<longrightarrow> dest x \<sqsubset> PC)" 

text {*
Definitions of the policies 
*}

constdefs
 strictPolicy :: "(ipv4,'b) Policy"
"strictPolicy \<equiv> deny_all ++ allow_all_from_to PC Internet"

 ProtocolPolicy :: "(ipv4,'b) Policy"
"ProtocolPolicy \<equiv> deny_all ++ allow_prots_from_to {http,ftp,imap} PC Internet"

 ProtocolPortPolicy :: "(ipv4,'b) Policy"
"ProtocolPortPolicy \<equiv> deny_all ++ allow_prots_from_port_to {(http,80::port),(ftp,24),(imap,21)} PC Internet"

 PortPolicy :: "(ipv4,'b) Policy"
"PortPolicy \<equiv> deny_all ++ allow_from_ports_to {80::port,24,21} PC Internet"


 ProtocolPolicyBig :: "(ipv4,'b) Policy"
"ProtocolPolicyBig \<equiv> deny_all ++ allow_prot_from_to http PC Internet ++ allow_prot_from_to ftp PC Internet++ allow_prot_from_to imap PC Internet"

 ProtocolPortPolicyBig :: "(ipv4,'b) Policy"
"ProtocolPortPolicyBig \<equiv> deny_all ++ (allow_prot_from_port_to http PC (80::port) Internet) ++ allow_prot_from_port_to ftp PC (24::port) Internet ++ allow_prot_from_port_to imap PC (21::port) Internet"

 PortPolicyBig :: "(ipv4,'b) Policy"
"PortPolicyBig \<equiv> deny_all ++ allow_from_port_to (80::port) PC Internet++ allow_from_port_to (24::port) PC Internet++ allow_from_port_to (21::port) PC Internet"


lemmas policyLemmas = strictPolicy_def ProtocolPolicy_def ProtocolPortPolicy_def PortPolicy_def PC_def Internet_def
ProtocolPolicyBig_def ProtocolPortPolicyBig_def PortPolicyBig_def

text{*

Testing the three policies

*}


test_spec "not_in_same_net x \<longrightarrow> FUT x = strictPolicy x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules IPv4Lemmas)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas IPv4Lemmas)

store_test_thm "strict"

gen_test_data "strict"

test_spec "not_in_same_net x \<longrightarrow> FUT x = ProtocolPolicy x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules IPv4Lemmas)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas IPv4Lemmas)
store_test_thm "protocol"

gen_test_data "protocol"


test_spec "not_in_same_net x \<longrightarrow> FUT x = ProtocolPortPolicy x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules IPv4Lemmas PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas IPv4Lemmas)
store_test_thm "protocolPort"


gen_test_data "protocolPort"

test_spec "not_in_same_net x \<longrightarrow> FUT x = PortPolicy x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules IPv4Lemmas PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas IPv4Lemmas)
store_test_thm "port"
gen_test_data "port"



(* BIG *)


test_spec "not_in_same_net x \<longrightarrow> FUT x = ProtocolPolicyBig x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules IPv4Lemmas PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas IPv4Lemmas)
store_test_thm "protocolBig"

gen_test_data "protocolBig"


test_spec "not_in_same_net x \<longrightarrow> FUT x = ProtocolPortPolicyBig x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules IPv4Lemmas PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas IPv4Lemmas)
store_test_thm "protocolPortBig"


gen_test_data "protocolPortBig"


test_spec "not_in_same_net x \<longrightarrow> FUT x = PortPolicyBig x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules IPv4Lemmas PortCombinators allow_from_port_to_def)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas IPv4Lemmas)

store_test_thm "portBig"


gen_test_data "portBig"



end