
theory SimpleDMZDatatype
imports "../../src/PacketFilter/PortCombinators" "../../src/FWTesting"
begin



datatype networks = Intranet | DMZ | Internet

types port = int

types nets = "networks \<times> port"

instance networks :: net ..


defs (overloaded)
src_port_def:  
  "src_port (x::(nets,'b) packet)  \<equiv> (snd o fst o snd o snd) x"
dest_port_def:
  "dest_port (x::(nets,'b) packet)   \<equiv> (snd o fst o snd o snd o snd) x"


constdefs
  intranet::"nets subnet"
  "intranet \<equiv> {{(a,b) . a=Intranet }}"

  dmz :: "nets subnet"
  "dmz \<equiv> {{(a,b). a=DMZ}}"

  internet :: "nets subnet"
  "internet \<equiv> {{(a,b).  a=Internet }}"


constdefs 
  Intranet_DMZ :: "(nets,'b) Rule"
  "Intranet_DMZ \<equiv> (allow_prots_from_to {smtp,imap} intranet dmz)"

  Intranet_Internet :: "(nets,'b) Rule"
  "Intranet_Internet \<equiv> allow_prot_from_to http intranet internet"
           
  Internet_DMZ :: "(nets,'b) Rule"
  "Internet_DMZ \<equiv> (allow_prots_from_to {http,smtp} internet dmz)"

constdefs 
  Intranet_DMZ_ProtocolPort :: "(nets,'b) Rule"
  "Intranet_DMZ_ProtocolPort \<equiv> (allow_prots_from_port_to {(smtp,21::port),(imap,24)} intranet dmz)"

  Intranet_Internet_ProtocolPort :: "(nets,'b) Rule"
  "Intranet_Internet_ProtocolPort \<equiv> allow_prot_from_port_to http intranet (80::port) internet"
           
  Internet_DMZ_ProtocolPort :: "(nets,'b) Rule"
  "Internet_DMZ_ProtocolPort \<equiv> (allow_prots_from_port_to {(http,80::port),(smtp,21)} internet dmz)"


constdefs 
  Intranet_DMZ_Port :: "(nets,'b) Rule"
  "Intranet_DMZ_Port \<equiv> (allow_from_ports_to {21::port,24} intranet dmz)"

  Intranet_Internet_Port :: "(nets,'b) Rule"
  "Intranet_Internet_Port \<equiv> allow_all_from_port_to intranet (80::port) internet"
           
  Internet_DMZ_Port :: "(nets,'b) Rule"
  "Internet_DMZ_Port \<equiv> (allow_from_ports_to {80::port,21} internet dmz)"

  
constdefs 
  policyProtocol :: "(nets, DummyContent) Policy"
 "policyProtocol \<equiv> deny_all ++ 
                  Intranet_Internet ++
                  Intranet_DMZ ++  
                  Internet_DMZ"

constdefs 
  policyProtocolPort :: "(nets, DummyContent) Policy"
 "policyProtocolPort \<equiv> deny_all ++ 
                  Intranet_Internet_ProtocolPort  ++
                  Intranet_DMZ_ProtocolPort ++  
                  Internet_DMZ_ProtocolPort"

constdefs 
  policyPort :: "(nets, DummyContent) Policy"
 "policyPort \<equiv> deny_all ++ 
                  Intranet_Internet_Port ++
                  Intranet_DMZ_Port ++  
                  Internet_DMZ_Port"

lemma protocol_is : "protocol (a,p,d,b,e) = p"
    by (simp add: protocol_def in_subnet_def)

lemma src_port : "src_port ((a,c,x,d,e)::(nets,'b) packet) = snd x"
  by (simp add: src_port_def in_subnet_def)

lemma dest_port : "dest_port ((a,c,d,x,e) ::(nets,'b) packet)= snd x"
  by (simp add: dest_port_def in_subnet_def)


lemmas PolicyLemmas =  policyProtocol_def dmz_def internet_def intranet_def
                       Intranet_Internet_def Intranet_DMZ_def 
                       Internet_DMZ_def dmz_def internet_def intranet_def
                       Intranet_Internet_ProtocolPort_def Intranet_DMZ_ProtocolPort_def 
                       Internet_DMZ_ProtocolPort_def policyProtocolPort_def
                       Intranet_Internet_Port_def Intranet_DMZ_Port_def 
                       Internet_DMZ_Port_def policyPort_def 
                       src_def dest_def in_subnet_def in_subnet_def protocol_is src_port dest_port

constdefs
 not_in_same_net :: "(nets,'b) packet \<Rightarrow> bool"
 "not_in_same_net x  \<equiv>  (src x \<sqsubset> internet \<longrightarrow> \<not> dest x \<sqsubset> internet) \<and> 
                     (src x \<sqsubset> intranet \<longrightarrow> \<not> dest x \<sqsubset> intranet) \<and> 
                     (src x \<sqsubset> dmz \<longrightarrow> \<not> dest x \<sqsubset> dmz)"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policyProtocol x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas stateless_rules PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas) 
store_test_thm "ProtocolPolicy"

gen_test_data "ProtocolPolicy"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policyProtocolPort x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas stateless_rules PortCombinators)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def  PolicyLemmas)  

store_test_thm "ProtocolPortPolicy"


gen_test_data "ProtocolPortPolicy"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policyPort x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas stateless_rules PortCombinators allow_all_from_port_to_def)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def PolicyLemmas)  
store_test_thm "PortPolicy"

gen_test_data "PortPolicy"


end
