
theory SimpleDMZWithoutPorts
imports "../../src/PacketFilter/IntegerNetwork" "../../src/PacketFilter/PolicyCombinators" "../../src/FWTesting"
begin


text{*

This is the Simple DMZ example. There are three subnetworks, with either none or some protocols allowed between then. 

The policy is always in its simplest form, but specified in three variants: 
  - The allowed protocols named
  - The allowed protocols named including their port number
  - The allowed protocols only specified through their port number


The used network model is the Integer model

*}



constdefs
  intranet::"IntegerNetwork subnet"
  "intranet \<equiv> {{a . (a > 2) \<and> (a < 5) }}"

  dmz :: "IntegerNetwork subnet"
  "dmz \<equiv> {{a. (a > 6) \<and> (a < 9)}}"

  internet :: "IntegerNetwork subnet"
  "internet \<equiv> {{a.   ( (\<not>(a > 2) \<or>  \<not>(a < 5) ) \<and> (\<not>(a > 6) \<or> \<not>(a < 9)))   }}"


constdefs 
  Intranet_DMZ :: "(IntegerNetwork,'b) Rule"
  "Intranet_DMZ \<equiv> (allow_prots_from_to {smtp,imap} intranet dmz)"

  Intranet_Internet :: "(IntegerNetwork,'b) Rule"
  "Intranet_Internet \<equiv> allow_prot_from_to http intranet internet"
           
  Internet_DMZ :: "(IntegerNetwork,'b) Rule"
  "Internet_DMZ \<equiv> (allow_prots_from_to {http,smtp} internet dmz)"
  
constdefs 
  policyProtocol :: "(IntegerNetwork, DummyContent) Policy"
 "policyProtocol \<equiv> deny_all ++ 
                  Intranet_Internet ++
                  Intranet_DMZ ++  
                  Internet_DMZ"


lemmas PolicyLemmas =  policyProtocol_def dmz_def internet_def intranet_def
                       Intranet_Internet_def Intranet_DMZ_def 
                       Internet_DMZ_def dmz_def internet_def intranet_def
                      
constdefs
 not_in_same_net :: "(IntegerNetwork,'b) packet \<Rightarrow> bool"
 "not_in_same_net x  \<equiv>  (src x \<sqsubset> internet \<longrightarrow> \<not> dest x \<sqsubset> internet) \<and> 
                     (src x \<sqsubset> intranet \<longrightarrow> \<not> dest x \<sqsubset> intranet) \<and> 
                     (src x \<sqsubset> dmz \<longrightarrow> \<not> dest x \<sqsubset> dmz)"


test_spec "not_in_same_net x \<longrightarrow> FUT x = policyProtocol x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas stateless_rules  IntegerLemmas src_def dest_def in_subnet_def)
apply (gen_test_cases "FUT")
apply (simp_all add: PolicyLemmas not_in_same_net_def IntegerLemmas src_def dest_def in_subnet_def) 
store_test_thm "ProtocolPolicyNoPorts"

testgen_params[iterations=50]


gen_test_data "ProtocolPolicyNoPorts"


end
