(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2008 ETH Zurich, Switzerland
 *               2008 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: four_subnets.thy 7470 2008-03-22 00:39:03Z brucker $ *)

theory 
  four_subnets
imports
  PolicyCombinators 
  FWTesting
begin

datatype Networks = One | Two | Three | Four

instance Networks::net .. 

constdefs
  one::"Networks subnet"
  "one \<equiv> {{One}}"
  two :: "Networks subnet"
  "two \<equiv> {{Two}}"
  three :: "Networks subnet"
  "three \<equiv> {{Three}}"
  four :: "Networks subnet"
  "four \<equiv> {{Four}}"


constdefs 
  NaivePolicy :: "(Networks,'b) Rule"
 "NaivePolicy \<equiv> deny_all ++
                deny_all_from_to two one ++ 
                allow_prot_from_to ftp two one ++ 
                deny_prot_from_to voip one two ++
                allow_prot_from_to imap two four ++ 
                allow_prot_from_to smtp three four ++ 
                deny_prot_from_to https three two ++
                allow_prot_from_to ntp three four ++
                allow_prot_from_to smtp one two"

constdefs
  SimplifiedPolicy :: "(Networks,'b) Rule"
 "SimplifiedPolicy \<equiv> deny_all ++
                     allow_prot_from_to ftp two one ++
                     allow_prots_from_to {voip,smtp} one two++
                     allow_prots_from_to {imap,https} two four ++
                     allow_prots_from_to {smtp,ntp} three four"
                    


text {* We only want to create test cases which are sent between the
  three main networks --- e.g. not between the mailserver and the
  dmz. Therefore, the constraint looks as follows. *}

constdefs
 not_in_same_net :: "(Networks,DummyContent) packet \<Rightarrow> bool"
 "not_in_same_net x  \<equiv>  (src x  \<noteq> dest x)"

constdefs
 not_in_same_net1 :: "(Networks,DummyContent) packet \<Rightarrow> bool"
 "not_in_same_net1 x  \<equiv>  (src x \<sqsubset> one \<longrightarrow> \<not> dest x \<sqsubset> one) \<and> 
                        (src x \<sqsubset> two \<longrightarrow> \<not> dest x \<sqsubset> two) \<and> 
                        (src x \<sqsubset> three \<longrightarrow> \<not> dest x \<sqsubset> three) \<and> 
                        (src x \<sqsubset> four \<longrightarrow> \<not> dest x \<sqsubset> four)"


lemmas PolicyLemmas =  NaivePolicy_def SimplifiedPolicy_def one_def two_def three_def four_def 
                   

test_spec "not_in_same_net x \<longrightarrow> FUT (x:: (Networks,DummyContent) packet) = SimplifiedPolicy x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas src_def dest_def in_subnet_def protocol_def stateless_rules )
apply (gen_test_cases "FUT")
apply simp_all
apply (simp_all add: not_in_same_net_def src_def dest_def in_subnet_def PolicyLemmas)
store_test_thm "Simplified"

test_spec "not_in_same_net x \<longrightarrow> FUT (x:: (Networks,DummyContent) packet) = NaivePolicy x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas src_def dest_def in_subnet_def protocol_def stateless_rules )
apply (gen_test_cases "FUT")
apply simp_all
apply (simp_all add: not_in_same_net_def src_def dest_def in_subnet_def PolicyLemmas)
store_test_thm "Naive"



end
