(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2008 ETH Zurich, Switzerland
 *               2008 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: DMZDatatype.thy 7470 2008-03-22 00:39:03Z brucker $ *)

theory 
  DMZDatatype
imports 
  PortCombinators 
  FWTesting
begin

datatype Networks = Intranet | Internet | Mail | Web | DMZ

instance Networks::net .. 

constdefs
  intranet::"Networks subnet"
  "intranet \<equiv> {{a. a= Intranet}}"
  dmz :: "Networks subnet"
  "dmz \<equiv> {{a. a= DMZ}}"
  mail :: "Networks subnet"
  "mail \<equiv> {{a. a=Mail}}"
  web :: "Networks subnet"
  "web \<equiv> {{a.  a=Web}}"

  internet :: "Networks subnet"
  "internet \<equiv> {{a. a= Internet}}"



constdefs 
  Intranet_mail :: "(Networks,'b) Rule"
  "Intranet_mail \<equiv> (allow_prots_from_to {smtp,imap} intranet mail)" 

  Intranet_Internet :: "(Networks,'b) Rule"
  "Intranet_Internet \<equiv> allow_prots_from_to {http,https} intranet internet"
     
  Internet_web :: "(Networks,'b) Rule"
  "Internet_web \<equiv> (allow_prots_from_to {http,https} internet web)"
          
  Internet_mail :: "(Networks,'b) Rule"
  "Internet_mail \<equiv>  (allow_prot_from_to smtp internet dmz)"


constdefs 
  policyProtocol :: "(Networks, DummyContent) Policy"
 "policyProtocol \<equiv> deny_all ++ 
                  Intranet_Internet ++ 
                  Intranet_mail ++  
                  Internet_mail ++ 
                  Internet_web"



text {* We only want to create test cases which are sent between the three main networks --- e.g. not between the mailserver and the dmz. Therefore, the constraint looks as follows. *}

constdefs
 not_in_same_net :: "(Networks,DummyContent) packet \<Rightarrow> bool"
 "not_in_same_net x  \<equiv>  (src x \<sqsubset> internet \<longrightarrow> \<not> dest x \<sqsubset> internet) \<and> 
                        (src x \<sqsubset> intranet \<longrightarrow> \<not> dest x \<sqsubset> intranet) \<and> 
                        (src x \<sqsubset> dmz \<longrightarrow> \<not> dest x \<sqsubset> dmz) \<and> 
                        (src x \<sqsubset> mail \<longrightarrow> ((dest x \<sqsubset> internet) \<or> (dest x \<sqsubset> intranet))) \<and> 
                        (src x \<sqsubset> web \<longrightarrow> ((dest x \<sqsubset> internet) \<or> (dest x \<sqsubset> intranet)))"

lemmas PolicyLemmas =  policyProtocol_def  dmz_def internet_def intranet_def mail_def web_def 
                       Intranet_Internet_def Intranet_mail_def 
                       Internet_web_def Internet_mail_def



test_spec "not_in_same_net x \<longrightarrow> FUT (x:: (Networks,DummyContent) packet) = policyProtocol x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas src_def dest_def in_subnet_def protocol_def stateless_rules )
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def src_def dest_def in_subnet_def PolicyLemmas)
store_test_thm "protocol"

end
