(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2008 ETH Zurich, Switzerland
 *               2008 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: PersonallFirewallIntWithoutPorts.thy 7411 2008-03-16 23:18:50Z brucker $ *)

theory 
  PersonalFirewallIntWithoutPorts
imports 
  IntegerNetwork
  PortCombinators 
  FWTesting
begin


text{*

The most basic firewall scenario; there is a personal PC on one side and the Internet on the other. 
There are two policies: the first one allows all traffic from the PC to the Internet and denies
all coming into the PC. The second policy only allows specific ports from the PC. This scenario
comes in three variants: the first one specifies the allowed protocols directly, the second together 
with their respective port numbers, the third one only with the port numbers.

*}



text{*
Definitions of the subnets 
*}

constdefs
 
 PC :: "(IntegerNetwork subnet)"
"PC \<equiv> {{3}}"

 Internet :: "IntegerNetwork subnet"
"Internet \<equiv> {{a. \<not> (a = 3)}}"



text{*
Definition of the testing constraint
*}

constdefs
 not_in_same_net :: "(IntegerNetwork,'b) packet \<Rightarrow> bool"
"not_in_same_net x \<equiv> (src x \<sqsubset> PC \<longrightarrow> dest x \<sqsubset> Internet) \<and> (src x \<sqsubset> Internet \<longrightarrow> dest x \<sqsubset> PC)" 

text {*
Definitions of the policies 
*}

constdefs
 strictPolicy :: "(IntegerNetwork,'b) Policy"
"strictPolicy \<equiv> deny_all ++ allow_all_from_to PC Internet"

 ProtocolPolicy :: "(IntegerNetwork,'b) Policy"
"ProtocolPolicy \<equiv> deny_all ++ allow_prots_from_to {http,ftp,imap} PC Internet"


 

lemmas policyLemmas = strictPolicy_def ProtocolPolicy_def PC_def Internet_def

text{*

Testing the three policies

*}


test_spec "not_in_same_net x \<longrightarrow> FUT x = strictPolicy x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules IntegerLemmas src_def dest_def in_subnet_def)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas IntegerLemmas src_def dest_def in_subnet_def)
store_test_thm "test"

test_spec "not_in_same_net x \<longrightarrow> FUT x = ProtocolPolicy x"
apply (prepareFWthm)
apply (simp add: policyLemmas stateless_rules IntegerLemmas src_def dest_def in_subnet_def)
apply (gen_test_cases "FUT")
apply (simp_all add: not_in_same_net_def policyLemmas IntegerLemmas src_def dest_def in_subnet_def)
store_test_thm "test"

end
