(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2008 ETH Zurich, Switzerland
 *               2008 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: rules-IntegerNetwork.thy 7470 2008-03-22 00:39:03Z brucker $ *)

theory 
  "rules-IntegerNetwork"
imports
  IntegerPairNetwork
  PolicyCombinators
  PortCombinators
  FWTesting
begin

constdefs
  workstation::"IntegerPair subnet"
  "workstation \<equiv> {{(a,b) . (a=3) }}"

  internet :: "IntegerPair subnet"
  "internet \<equiv> {{(a,b).  \<not> (a =3)}}"

constdefs
  not_in_same_net :: "(IntegerPair,DummyContent) packet \<Rightarrow> bool"
  "not_in_same_net x  \<equiv>  (src x \<sqsubset> workstation \<longrightarrow> ( (dest x \<sqsubset> internet))) \<and> 
  (src x \<sqsubset> internet \<longrightarrow>  dest x \<sqsubset> workstation)"

constdefs
   policy_00 :: "(IntegerPair, DummyContent) Policy"
  "policy_00 \<equiv> deny_all"

   policy_01 :: "(IntegerPair, DummyContent) Policy"
  "policy_01 \<equiv> policy_00 ++ allow_prot_from_to http workstation internet"

   policy_02 :: "(IntegerPair, DummyContent) Policy"
  "policy_02 \<equiv> policy_01 ++ allow_prot_from_to ftp workstation internet"

   policy_03 :: "(IntegerPair, DummyContent) Policy"
  "policy_03 \<equiv> policy_02 ++ allow_prot_from_to imap workstation internet"

   policy_04 :: "(IntegerPair, DummyContent) Policy"
  "policy_04 \<equiv> policy_03 ++ allow_prot_from_to udp workstation internet "

   policy_05 :: "(IntegerPair, DummyContent) Policy"
  "policy_05 \<equiv> policy_04 ++ allow_prot_from_to IP workstation internet"

   policy_06 :: "(IntegerPair, DummyContent) Policy"
  "policy_06 \<equiv> policy_05 ++ allow_prot_from_to icmp workstation internet"

   policy_07 :: "(IntegerPair, DummyContent) Policy"
  "policy_07 \<equiv> policy_06 ++ allow_prot_from_to https workstation internet"

   policy_08 :: "(IntegerPair, DummyContent) Policy"
  "policy_08 \<equiv> policy_07 ++ allow_prot_from_to smtp workstation internet"

   policy_09 :: "(IntegerPair, DummyContent) Policy"
  "policy_09 \<equiv> policy_08 ++ allow_prot_from_to imaps workstation internet"

   policy_10 :: "(IntegerPair, DummyContent) Policy"
  "policy_10 \<equiv> policy_09 ++ allow_prot_from_to unknown workstation internet"


lemmas UnfoldPolicy =  workstation_def internet_def policy_00_def 
                       policy_01_def policy_02_def policy_03_def policy_04_def policy_05_def
                       policy_06_def policy_07_def policy_08_def policy_09_def policy_10_def

lemmas UnfoldNetworkAndPolicy = UnfoldPolicy IntegerNetworkLemmas PortCombinators stateless_rules

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_00 x"
  apply (prepareFWthm, simp add: UnfoldNetworkAndPolicy)
  apply (gen_test_cases "FUT")
  apply (simp_all add: UnfoldNetworkAndPolicy not_in_same_net_def)  
  store_test_thm "policy_00"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_01 x"
  apply (prepareFWthm, simp add: UnfoldNetworkAndPolicy)
  apply (gen_test_cases "FUT")
  apply (simp_all add: UnfoldNetworkAndPolicy not_in_same_net_def)  
  store_test_thm "policy_01"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_02 x"
  apply (prepareFWthm, simp add: UnfoldNetworkAndPolicy)
  apply (gen_test_cases "FUT")
  apply (simp_all add: UnfoldNetworkAndPolicy not_in_same_net_def)  
  store_test_thm "policy_02"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_03 x"
  apply (prepareFWthm, simp add: UnfoldNetworkAndPolicy)
  apply (gen_test_cases "FUT")
  apply (simp_all add: UnfoldNetworkAndPolicy not_in_same_net_def)  
  store_test_thm "policy_03"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_04 x"
  apply (prepareFWthm, simp add: UnfoldNetworkAndPolicy)
  apply (gen_test_cases "FUT")
  apply (simp_all add: UnfoldNetworkAndPolicy not_in_same_net_def)  
  store_test_thm "policy_04"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_05 x"
  apply (prepareFWthm, simp add: UnfoldNetworkAndPolicy)
  apply (gen_test_cases "FUT")
  apply (simp_all add: UnfoldNetworkAndPolicy not_in_same_net_def)  
  store_test_thm "policy_05"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_06 x"
  apply (prepareFWthm, simp add: UnfoldNetworkAndPolicy)
  apply (gen_test_cases "FUT")
  apply (simp_all add: UnfoldNetworkAndPolicy not_in_same_net_def)  
  store_test_thm "policy_06"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_07 x"
  apply (prepareFWthm, simp add: UnfoldNetworkAndPolicy)
  apply (gen_test_cases "FUT")
  apply (simp_all add: UnfoldNetworkAndPolicy not_in_same_net_def)  
  store_test_thm "policy_07"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_08 x"
  apply (prepareFWthm, simp add: UnfoldNetworkAndPolicy)
  apply (gen_test_cases "FUT")
  apply (simp_all add: UnfoldNetworkAndPolicy not_in_same_net_def)  
  store_test_thm "policy_08"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_09 x"
  apply (prepareFWthm, simp add: UnfoldNetworkAndPolicy)
  apply (gen_test_cases "FUT")
  apply (simp_all add: UnfoldNetworkAndPolicy not_in_same_net_def)  
  store_test_thm "policy_09"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policy_10 x"
  apply (prepareFWthm, simp add: UnfoldNetworkAndPolicy)
  apply (gen_test_cases "FUT")
  apply (simp_all add: UnfoldNetworkAndPolicy not_in_same_net_def)  
  store_test_thm "policy_10"

end
