(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * ROOT.ML --- main file
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2008 Achim D. Brucker, Germany
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: SimpleDMZDatatype.thy 7431 2008-03-17 20:26:34Z brucker $ *)
theory 
  SimpleDMZDatatype
imports
  PortCombinators
  FWTesting
begin



datatype networks = Intranet | DMZ | Internet

types port = int

types nets = "networks \<times> port"

instance networks :: net ..


defs (overloaded)
src_port_def:  
  "src_port (x::(nets,'b) packet)  \<equiv> (snd o fst o snd o snd) x"
dest_port_def:
  "dest_port (x::(nets,'b) packet)   \<equiv> (snd o fst o snd o snd o snd) x"


constdefs
  intranet::"nets subnet"
  "intranet \<equiv> {{(a,b) . a=Intranet }}"

  dmz :: "nets subnet"
  "dmz \<equiv> {{(a,b). a=DMZ}}"

  internet :: "nets subnet"
  "internet \<equiv> {{(a,b).  a=Internet }}"


constdefs 
  Intranet_DMZ :: "(nets,'b) Rule"
  "Intranet_DMZ \<equiv> (allow_prots_from_to {smtp,imap} intranet dmz)"

  Intranet_Internet :: "(nets,'b) Rule"
  "Intranet_Internet \<equiv> allow_prot_from_to http intranet internet"
           
  Internet_DMZ :: "(nets,'b) Rule"
  "Internet_DMZ \<equiv> (allow_prots_from_to {http,smtp} internet dmz)"

constdefs 
  Intranet_DMZ_ProtocolPort :: "(nets,'b) Rule"
  "Intranet_DMZ_ProtocolPort \<equiv> (allow_prots_from_port_to {(smtp,21::port),(imap,24)} intranet dmz)"

  Intranet_Internet_ProtocolPort :: "(nets,'b) Rule"
  "Intranet_Internet_ProtocolPort \<equiv> allow_prot_from_port_to http intranet (80::port) internet"
           
  Internet_DMZ_ProtocolPort :: "(nets,'b) Rule"
  "Internet_DMZ_ProtocolPort \<equiv> (allow_prots_from_port_to {(http,80::port),(smtp,21)} internet dmz)"


constdefs 
  Intranet_DMZ_Port :: "(nets,'b) Rule"
  "Intranet_DMZ_Port \<equiv> (allow_from_ports_to {21::port,24} intranet dmz)"

  Intranet_Internet_Port :: "(nets,'b) Rule"
  "Intranet_Internet_Port \<equiv> allow_all_from_port_to intranet (80::port) internet"
           
  Internet_DMZ_Port :: "(nets,'b) Rule"
  "Internet_DMZ_Port \<equiv> (allow_from_ports_to {80::port,21} internet dmz)"

  
constdefs 
  policyProtocol :: "(nets, DummyContent) Policy"
 "policyProtocol \<equiv> deny_all ++ 
                  Intranet_Internet ++
                  Intranet_DMZ ++  
                  Internet_DMZ"

constdefs 
  policyProtocolPort :: "(nets, DummyContent) Policy"
 "policyProtocolPort \<equiv> deny_all ++ 
                  Intranet_Internet_ProtocolPort  ++
                  Intranet_DMZ_ProtocolPort ++  
                  Internet_DMZ_ProtocolPort"

constdefs 
  policyPort :: "(nets, DummyContent) Policy"
 "policyPort \<equiv> deny_all ++ 
                  Intranet_Internet_Port ++
                  Intranet_DMZ_Port ++  
                  Internet_DMZ_Port"

lemma protocol_is : "protocol (a,p,d,b,e) = p"
    by (simp add: protocol_def in_subnet_def)

lemma src_port : "src_port ((a,c,x,d,e)::(nets,'b) packet) = snd x"
  by (simp add: src_port_def in_subnet_def)

lemma dest_port : "dest_port ((a,c,d,x,e) ::(nets,'b) packet)= snd x"
  by (simp add: dest_port_def in_subnet_def)


lemmas PolicyLemmas =  policyProtocol_def dmz_def internet_def intranet_def
                       Intranet_Internet_def Intranet_DMZ_def 
                       Internet_DMZ_def dmz_def internet_def intranet_def
                       Intranet_Internet_ProtocolPort_def Intranet_DMZ_ProtocolPort_def 
                       Internet_DMZ_ProtocolPort_def policyProtocolPort_def
                       Intranet_Internet_Port_def Intranet_DMZ_Port_def 
                       Internet_DMZ_Port_def policyPort_def 
                       src_def dest_def in_subnet_def in_subnet_def protocol_is src_port dest_port

constdefs
 not_in_same_net :: "(nets,'b) packet \<Rightarrow> bool"
 "not_in_same_net x  \<equiv>  (src x \<sqsubset> internet \<longrightarrow> \<not> dest x \<sqsubset> internet) \<and> 
                     (src x \<sqsubset> intranet \<longrightarrow> \<not> dest x \<sqsubset> intranet) \<and> 
                     (src x \<sqsubset> dmz \<longrightarrow> \<not> dest x \<sqsubset> dmz)"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policyProtocol x"
  apply (prepareFWthm)
  apply (simp add: PolicyLemmas stateless_rules PortCombinators)
  apply (gen_test_cases "FUT")
  apply (simp_all add: not_in_same_net_def PolicyLemmas) 
  store_test_thm "ProtocolPolicy"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policyProtocolPort x"
  apply (prepareFWthm)
  apply (simp add: PolicyLemmas stateless_rules PortCombinators)
  apply (gen_test_cases "FUT")
  apply (simp_all add: not_in_same_net_def  PolicyLemmas)  
  store_test_thm "ProtocolPortPolicy"

test_spec "not_in_same_net x \<longrightarrow> FUT x = policyPort x"
  apply (prepareFWthm)
  apply (simp add: PolicyLemmas stateless_rules PortCombinators allow_all_from_port_to_def)
  apply (gen_test_cases "FUT")
  apply (simp_all add: not_in_same_net_def PolicyLemmas)  
  store_test_thm "PortPolicy"



end
