(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2008 ETH Zurich, Switzerland
 *               2008 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: SimpleDMZWithoutPorts.thy 7470 2008-03-22 00:39:03Z brucker $ *)

theory 
  SimpleDMZWithoutPorts
imports 
  IntegerNetwork
  PolicyCombinators
  FWTesting
begin


text{*

This is the Simple DMZ example. There are three subnetworks, with either none or some protocols allowed between then. 

The policy is always in its simplest form, but specified in three variants: 
  - The allowed protocols named
  - The allowed protocols named including their port number
  - The allowed protocols only specified through their port number


The used network model is the Integer model

*}



constdefs
  intranet::"IntegerNetwork subnet"
  "intranet \<equiv> {{a . (a > 2) \<and> (a < 5) }}"

  dmz :: "IntegerNetwork subnet"
  "dmz \<equiv> {{a. (a > 6) \<and> (a < 9)}}"

  internet :: "IntegerNetwork subnet"
  "internet \<equiv> {{a.   ( (\<not>(a > 2) \<or>  \<not>(a < 5) ) \<and> (\<not>(a > 6) \<or> \<not>(a < 9)))   }}"


constdefs 
  Intranet_DMZ :: "(IntegerNetwork,'b) Rule"
  "Intranet_DMZ \<equiv> (allow_prots_from_to {smtp,imap} intranet dmz)"

  Intranet_Internet :: "(IntegerNetwork,'b) Rule"
  "Intranet_Internet \<equiv> allow_prot_from_to http intranet internet"
           
  Internet_DMZ :: "(IntegerNetwork,'b) Rule"
  "Internet_DMZ \<equiv> (allow_prots_from_to {http,smtp} internet dmz)"
  
constdefs 
  policyProtocol :: "(IntegerNetwork, DummyContent) Policy"
 "policyProtocol \<equiv> deny_all ++ 
                  Intranet_Internet ++
                  Intranet_DMZ ++  
                  Internet_DMZ"


lemmas PolicyLemmas =  policyProtocol_def dmz_def internet_def intranet_def
                       Intranet_Internet_def Intranet_DMZ_def 
                       Internet_DMZ_def dmz_def internet_def intranet_def
                      
constdefs
 not_in_same_net :: "(IntegerNetwork,'b) packet \<Rightarrow> bool"
 "not_in_same_net x  \<equiv>  (src x \<sqsubset> internet \<longrightarrow> \<not> dest x \<sqsubset> internet) \<and> 
                     (src x \<sqsubset> intranet \<longrightarrow> \<not> dest x \<sqsubset> intranet) \<and> 
                     (src x \<sqsubset> dmz \<longrightarrow> \<not> dest x \<sqsubset> dmz)"


test_spec "not_in_same_net x \<longrightarrow> FUT x = policyProtocol x"
apply (prepareFWthm)
apply (simp add: PolicyLemmas stateless_rules  IntegerLemmas src_def dest_def in_subnet_def)
apply (gen_test_cases "FUT")
apply (simp_all add: PolicyLemmas not_in_same_net_def IntegerLemmas src_def dest_def in_subnet_def) 
store_test_thm "ProtocolPolicyNoPorts"

end
