(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * --- 
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2008 ETH Zurich, Switzerland
 *               2008 Achim D. Brucker, Germany 
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: PersonalFirewallDatatype.thy 7435 2008-03-18 00:35:38Z brucker $ *)

theory 
  PersonalFirewallDatatype
imports 
  DatatypeTwoNets
  PortCombinators 
  FWTesting
begin

constdefs
 PC :: "(DatatypeTwoNets subnet)"
"PC \<equiv> {{(a,b). a = one}}"

 Internet :: "DatatypeTwoNets subnet"
"Internet \<equiv> {{(a,b). a = two}}"

constdefs
 not_in_same_net :: "(DatatypeTwoNets,'b) packet \<Rightarrow> bool"
"not_in_same_net x \<equiv> (src x \<sqsubset> PC \<longrightarrow> dest x \<sqsubset> Internet) \<and> (src x \<sqsubset> Internet 
                     \<longrightarrow> dest x \<sqsubset> PC)" 


constdefs
 ProtocolPolicyOptimized :: "(DatatypeTwoNets,'b) Policy"
"ProtocolPolicyOptimized \<equiv> deny_all 
                           ++ allow_prots_from_to {http,ftp,imap} PC Internet"

 ProtocolPolicy :: "(DatatypeTwoNets,'b) Policy"
"ProtocolPolicy \<equiv> deny_all ++ allow_prot_from_to http PC Internet 
                           ++ allow_prot_from_to ftp PC Internet
                           ++ allow_prot_from_to imap PC Internet"


lemmas policyLemmas = ProtocolPolicy_def PC_def Internet_def 

test_spec "not_in_same_net x \<longrightarrow> FUT x = ProtocolPolicy x"
  apply (prepareFWthm)
  apply (simp add: policyLemmas stateless_rules DatatypeTwoNetsLemmas PortCombinators)
  apply (gen_test_cases "FUT")
  apply (simp_all add: not_in_same_net_def policyLemmas DatatypeTwoNetsLemmas)
  store_test_thm "PersonalFW_naive"

test_spec "not_in_same_net x \<longrightarrow> FUT x = ProtocolPolicy x"
  apply (prepareFWthm)
  apply (simp add: policyLemmas)
  apply (simplifyPolicy)
  apply (simp add: policyLemmas stateless_rules DatatypeTwoNetsLemmas PortCombinators)
  apply (gen_test_cases "FUT")
  apply (simp_all add: not_in_same_net_def policyLemmas DatatypeTwoNetsLemmas)
  store_test_thm "PersonalFW_simplified"

end
