theory DatatypeThreeNets
imports NetworkCore 
begin


text {* A simple scenario where the networks are modelled as elements of a datatype. In this example, there are three networks: a DMZ, an Intranet, and an Internet. 
The adresses also have ports.

 *}

 
datatype networks = DMZ |  INTRANET | INTERNET



instance networks :: net ..


types  
  port    =  "int"
  DatatypeThreeNets =  "(networks \<times> port)"


text{* 
  Analogous to the packet definition, we define projectors to access
  the ports of a packet directly. 
*}
constdefs
  src_port   :: "(DatatypeThreeNets, 'b) packet \<Rightarrow> port"
  "src_port  \<equiv> snd o fst o snd o snd"

  dest_port   :: "(DatatypeThreeNets, 'b) packet \<Rightarrow> port"
  "dest_port  \<equiv> snd o fst o snd o snd o snd"



text{* 
  As a help for later we also define constructors which take an
  address or an IP and return a subnet with all addresses having the
  same ipv4\_ip. 
*}
constdefs 
  subnet_of :: "DatatypeThreeNets  \<Rightarrow> DatatypeThreeNets subnet"
  "subnet_of adr \<equiv> {{(a,b). (a = fst adr)}}"
 

text{* 
  This is everything we need to model an address. Other address
  representations could as easily be added. For example we could write
  a similar theory to model IPv6 addresses. The place for the
  respective theory is depicted in the session graph in the appendix
  on page \pageref{fig:session_graph}. 
*}

lemma in_subnet: 
  "((((a), e)) \<sqsubset> {{((x),y). P x y}}) = (P a e)"
  by (simp add: in_subnet_def)

lemma src_in_subnet : 
  "((src(q,w,((a),e),r,t)) \<sqsubset> {{((x1),y). P x1 y}}) = (P a e)"
  by (simp add: in_subnet_def in_subnet src_def)

lemma dest_in_subnet : 
  "((dest (q,w,r,((a),e),t)) \<sqsubset> {{((x1),y). P x1 y}}) = (P a e)"
  by (simp add: in_subnet_def in_subnet dest_def)

lemma protocol_is : "protocol (a,p,d,b,e) = p"
    by (simp add: protocol_def in_subnet)

lemma src_port : "src_port (a,c,x,d,e) = snd x"
  by (simp add: src_port_def in_subnet)

lemma dest_port : "dest_port (a,c,d,x,e) = snd x"
  by (simp add: dest_port_def in_subnet)


lemmas DatatypeThreeNetsLemmas = in_subnet src_in_subnet dest_in_subnet protocol_is src_port dest_port
end
