theory DatatypeTwoNets
imports NetworkCore
begin


text {* A simple scenario where the networks are modelled as elements of a datatype. In this example, there are two networks, simply called ONE and TWO.
 
The adresses also have ports.

 *}

 
datatype networks = one | two


instance networks :: net ..


types  
  port    =  "int"
  DatatypeTwoNets =  "(networks \<times> port)"

(* instance port < port .. *)

defs (overloaded)
src_port_def:  
  "src_port (x::(DatatypeTwoNets,'b) packet)  \<equiv> (snd o fst o snd o snd) x"
dest_port_def:
  "dest_port (x::(DatatypeTwoNets,'b) packet)   \<equiv> (snd o fst o snd o snd o snd) x"



text{* 
  As a help for later we also define constructors which take an
  address or an IP and return a subnet with all addresses having the
  same ipv4\_ip. 
*}
constdefs 
  subnet_of :: "DatatypeTwoNets  \<Rightarrow> DatatypeTwoNets subnet"
  "subnet_of adr \<equiv> {{(a,b). (a = fst adr)}}"
 

text{* 
  This is everything we need to model an address. Other address
  representations could as easily be added. For example we could write
  a similar theory to model IPv6 addresses. The place for the
  respective theory is depicted in the session graph in the appendix
  on page \pageref{fig:session_graph}. 
*}

lemma in_subnet: 
  "((((a), e)) \<sqsubset> {{((x),y). P x y}}) = (P a e)"
  by (simp add: in_subnet_def)

lemma src_in_subnet : 
  "((src(q,w,((a),e),r,t)) \<sqsubset> {{((x1),y). P x1 y}}) = (P a e)"
  by (simp add: in_subnet_def in_subnet src_def)

lemma dest_in_subnet : 
  "((dest (q,w,r,((a),e),t)) \<sqsubset> {{((x1),y). P x1 y}}) = (P a e)"
  by (simp add: in_subnet_def in_subnet dest_def)

lemma protocol_is : "protocol (a,p,d,b,e) = p"
    by (simp add: protocol_def in_subnet)

lemma src_port : "src_port (a,c,x::DatatypeTwoNets,d,e) = snd x"
  by (simp add: src_port_def in_subnet)

lemma dest_port : "dest_port (a,c,d,x::DatatypeTwoNets,e) = snd x"
  by (simp add: dest_port_def in_subnet)


lemmas DatatypeTwoNetsLemmas = in_subnet src_in_subnet dest_in_subnet protocol_is src_port dest_port
end
