(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * IPv4.thy --- modelling ipv4 addresses
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2007, ETH Zurich, Switzerland
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: IPv4.thy 6651 2007-07-02 05:17:53Z brucker $ *)

header{* IPv4 *}
theory IntegerNetwork
imports NetworkCore
begin


text {* A simple theory where networks are modelled as one integer. There are no ports here *}



types 
  IntegerNetwork = "int" 




lemma in_subnet: 
  "(a \<sqsubset> {{x. P x}}) = (P a)"
  by (simp add: in_subnet_def)

lemma src_in_subnet: 
  "((src(q,w,a,r,t)) \<sqsubset> {{x. P q w x r t}}) = (P q w a r t)"
  by (simp add: in_subnet_def in_subnet src_def)

lemma dest_in_subnet: 
  "((dest (q,w,r,a,t)) \<sqsubset> {{x. P q w r x t}}) = (P q w r a t)"
  by (simp add: in_subnet_def in_subnet dest_def)

lemma src_in_subnet2: 
  "((src(q,w,a,r,t)) \<sqsubset> {{m}}) =  (a = m)"
  by (simp add: in_subnet_def in_subnet src_def)

lemma dest_in_subnet2: 
  "((dest (q,w,r,a,t)) \<sqsubset> {{m}}) = (a = m)"
  by (simp add: in_subnet_def in_subnet dest_def)


lemma protocol_is: "protocol (a,p,d,b,e) = p"
    by (simp add: protocol_def in_subnet)


lemma src_is: "src (a,p,d,b,e) = d"
    by (simp add: src_def in_subnet)

lemma dest_is: "dest (a,p,d,b,e) = b"
    by (simp add: dest_def in_subnet)


lemmas IntegerLemmas = in_subnet src_in_subnet dest_in_subnet  protocol_is src_def dest_def in_subnet_def

end
