(*****************************************************************************
 * HOL-TestGen --- theorem-prover based test case generation
 *                 http://www.brucker.ch/projects/hol-testgen/
 *                                                                            
 * IPv4.thy --- modelling ipv4 addresses
 * This file is part of HOL-TestGen.
 *
 * Copyright (c) 2005-2007, ETH Zurich, Switzerland
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: IPv4.thy 6651 2007-07-02 05:17:53Z brucker $ *)

header{* IPv4 *}
theory IntegerPairNetwork
imports NetworkCore
begin


text {* A simple theory where networks are modelled as a pair of Integers. The first one is the adress, the second one the port. *}


types 
  address = int
  port = int
  IntegerPair = "address \<times> port" 


(*
instance port < port .. 
*)

defs (overloaded)
 src_port_def: "src_port (x::(IntegerPair,'b) packet)  \<equiv> (snd o fst o snd o snd) x"

 dest_port_def: "dest_port  (x::(IntegerPair,'b) packet)  \<equiv> (snd o fst o snd o snd o snd) x"



lemma in_subnet: 
  "((((a), e)) \<sqsubset> {{((x1),y). P x1 y}}) = (P a e)"
  by (simp add: in_subnet_def)

lemma src_in_subnet: 
  "((src(q,w,((a),e),r,t)) \<sqsubset> {{((x1),y). P x1 y}}) = (P a e)"
  by (simp add: in_subnet_def in_subnet src_def)

lemma dest_in_subnet: 
  "((dest (q,w,r,((a),e),t)) \<sqsubset> {{((x1),y). P x1 y}}) = (P a e)"
  by (simp add: in_subnet_def in_subnet dest_def)

lemma protocol_is: "protocol (a,p,d,b,e) = p"
    by (simp add: protocol_def in_subnet)

lemma src_port: "src_port (a,c,x::IntegerPair,d,e) = snd x"
  by (simp add: src_port_def in_subnet)

lemma dest_port: "dest_port (a,c,d,x::IntegerPair,e) = snd x"
  by (simp add: dest_port_def in_subnet)



lemmas IntegerNetworkLemmas = in_subnet src_in_subnet dest_in_subnet protocol_is src_port dest_port

end
