theory PolicySimplification
imports PolicyCombinators PortCombinators 
begin

text{*
In dieser Theorie werden einige Experimente bezueglich Policy simplifizieren durchgefuehrt. 

*}


text{*
Was fuer Simplifikationen sind ueberhaupt moeglich, bzw. sinnvoll:

 - Rules, die von einer anderen, groesseren abgedeckt werden zu streichen. 
 - Mehrere Rules fuer dieselben src und dest zu einer einzigen zusammenfassen.

Das sind meiner Meinung nach die wichtigsten. Was gaebe es sonst noch?
*}


 
text{*

Zuerst zur ersten Kategorie: 

Diese Lemmas sind relativ trivial und auch entsprechend einfach zu beweisen. Man sieht jedoch schon hier, dass eine grosse Menge von Lemmas noetig sein werden, um eine vollstaendige PolicySimplification Theorie zu haben. Es gibt halt sehr viele Policy-Kombinatoren\<dots>
*}

lemma p1a: "deny_all ++ deny_all_from_to A B = deny_all"
apply (rule ext)
apply (simp add: stateless_rules)
done

lemma p1b: "deny_all ++ deny_prot_from_to p A B  = deny_all"
apply (rule ext)
apply (simp add: stateless_rules)
done


lemma p1c: "allow_all ++ allow_all_from_to A B = allow_all"
apply (rule ext)
apply (simp add: stateless_rules)
done

lemma p1d: "allow_all ++ allow_prot_from_to p A B  = allow_all"
apply (rule ext)
apply (simp add: stateless_rules)
done

lemma p1e: "allow_all ++ allow_prot_from_port_to p A po B  = allow_all"
apply (rule ext)
apply (simp add: PortCombinators stateless_rules)
done

lemma p1f: "allow_all ++ allow_from_port_to p A B  = allow_all"
apply (rule ext)
apply (simp add: stateless_rules PortCombinators )
done


(* Dies sind nur die wichtigsten. Es gibt natuerlich noch viele mehr, wie z.b. die folgende: *)

lemma p1i: "allow_all_from_to A B ++ allow_prot_from_to p A B = allow_all_from_to A B"
apply (rule ext)
apply (simp add: stateless_rules)
done


text{*
Die naechste Kategorie von Lemmas, fasst  aehnliche Regeln zu einer zusammen.
 
Dies verkleinert zwar die Policy, wieviel es jedoch bei der Testfallgenerierung wirklich bringt, muss ich noch ausfuehrlicher testen. Erste Experimente sind aber ermutigend.

Auch hier wird deutlich, dass eine vollstaendige Theorie sehr gross sein wird.

 *}


(* Fuer allow_prots_from_to *)

lemma p2a1: "allow_prot_from_to p A B ++ allow_prot_from_to q A B = allow_prots_from_to {q,p} A B"
apply (rule ext)
apply (simp add: stateless_rules)
done


lemma p2a2: "allow_prots_from_to P A B ++ allow_prot_from_to q A B = allow_prots_from_to (insert q P) A B"
apply (rule ext)
apply (simp add: stateless_rules)
done

lemma p2a3: "allow_prot_from_to q A B ++ allow_prots_from_to P A B = allow_prots_from_to (insert q P) A B"
apply (rule ext)
apply (simp add: stateless_rules)
done

(* Fuer allow_prots_from_to_port *)





text{* Analoges muesste man natuerlich beispielsweise fuer die deny Kombinatoren machen. *}


text{* Die Lemmas, die bis jetzt eingefuehrt wurden, sind vermutlich so das Kernstueck einer PolicySimplifikation. Um sie "anwendbar" zu machen, braucht es noch einige zusaetzliche Lemmas, z.b. bezueglich der Kommutativitaet und Assoziativitaet von Regeln. Viele dieser Lemmas gelten eigentlich fuer alle Maps und sind nicht Firewall spezifisch. *}


(* Es gilt Assoziativitaet. *)

lemma map_assoc: "(A ++ B) ++ C = (A ++ (B ++ C))" 
apply auto
done


(* Die naechsten vier Lemmas sind nur Hilfslemmas. *)

lemma p3a: "\<lbrakk>x \<in> dom A; x \<notin> dom B\<rbrakk> \<Longrightarrow> (A ++ B)x = (B ++ A)x"
apply (simp add: dom_def map_add_def)
apply auto
done

lemma p3b: "\<lbrakk>x \<notin> dom A; x \<in> dom B\<rbrakk> \<Longrightarrow> (A ++ B)x = (B ++ A)x"
apply (simp add: dom_def map_add_def)
apply auto
done
lemma p3c: "\<lbrakk>x \<notin> dom A; x \<notin> dom B\<rbrakk> \<Longrightarrow> (A ++ B)x = (B ++ A)x"
apply (simp add: dom_def map_add_def)
done
lemma p3d: "A \<inter> B = {} \<Longrightarrow> (x \<in> A \<and> x \<notin> B) \<or> (x \<notin> A \<and> x \<in>  B) \<or> (x \<notin> A \<and> x \<notin> B)"
apply auto
done


(* Dieses Lemma ist sehr wichtig: Es besagt, dass disjunkte Regeln kommutieren. *)
lemma map_comm: "dom A \<inter> dom B = {} \<Longrightarrow> A ++ B = B ++ A"
apply (rule ext)
apply (drule_tac x = x in p3d )
apply (case_tac "x \<in> dom A \<and> x \<notin> dom B")
apply simp
apply (rule p3a)
apply simp_all
apply (case_tac "x \<notin> dom A \<and> x \<in>  dom B")
apply (rule p3b)
apply simp_all
apply (rule p3c)
apply simp_all
done


(* Nun noch einige zusaetzlich Kommutativitaets Lemmas. Wohl nicht alle noetig, z.T. aber nuetzlich. *)

lemma p3e: "dom A \<inter> dom B = {} \<Longrightarrow> A ++ B ++ Z = B ++ A ++ Z"
apply (subst map_comm)
apply simp_all
done

lemma p3f: "dom A \<inter> dom B = {} \<Longrightarrow> Z ++ A ++ B = Z ++ B ++ A"
apply (subst map_assoc)
apply (subst map_assoc)
apply (subst map_comm) back 
apply simp_all 
done


lemma p3g: "dom A \<inter> dom B = {} \<Longrightarrow> Y ++ A ++ B ++ Z = Y ++ B ++ A ++ Z"
apply (subst map_assoc)
apply (subst map_assoc) back
apply (subst map_comm) back 
apply simp_all
done 






text{*

Nun einige Beispiele, welche die Theorie testen. Diese koennen dann natuerlich beim simplifizieren auch nuetzlich sein. Eine vollstaendige Liste mit allen moeglichen Kombinationen waere aber vermutlich zu umfangreich.
*}

(* Als erstes eine Definition die besagt, dass zwei Teilnetze disjunkt sind.  Damit lassen sich die folgenden Lemmas beweisen, ohne dass konrete Adressen angegeben werden muessen.
*)

constdefs 
  NetsDistinct :: "'a subnet \<Rightarrow> 'a subnet \<Rightarrow> bool"
 "NetsDistinct a b \<equiv> \<forall> x y. x \<in> a \<and> y \<in> b \<longrightarrow> x \<inter> y = {}"

(* Jetzt also einige Beispiele: *)

lemma p4a: "deny_all_from_to A B ++ deny_all_from_to C D = deny_all_from_to C D ++ deny_all_from_to A B"
apply (rule ext)
apply (simp add: stateless_rules)
done

lemma p4b: "NetsDistinct A C \<Longrightarrow> dom (deny_all_from_to A B) \<inter> dom (deny_all_from_to C D) = {}"
apply (simp only: deny_all_from_to_def)
apply (simp add: NetsDistinct_def)
apply (simp add: src_def dest_def)
apply (simp add: in_subnet_def)
apply (auto)
done



lemma p4c: "NetsDistinct A C \<Longrightarrow> deny_prots_from_to P A B ++ deny_prots_from_to Q C D = deny_prots_from_to Q C D ++ deny_prots_from_to P A B"
apply (simp add: NetsDistinct_def)
apply (subgoal_tac "dom (deny_prots_from_to P A B) \<inter> dom (deny_prots_from_to Q C D) = {}")
apply (subst map_comm)
apply simp_all
apply (simp add: in_subnet_def protocol_def src_def dest_def stateless_rules dom_def)
apply auto 
done


lemma p4d: "NetsDistinct A C \<Longrightarrow> deny_prots_from_to P A B ++ deny_prots_from_to Q C D = deny_prots_from_to Q C D ++ deny_prots_from_to P A B"
apply (simp add: NetsDistinct_def)
apply (subgoal_tac "dom (deny_prots_from_to P A B) \<inter> dom (deny_prots_from_to Q C D) = {}")
apply (subst map_comm)
apply simp_all
apply (simp add: src_def dest_def stateless_rules dom_def deny_all_def protocol_def in_subnet_def)
apply auto 
done


text {*  Nun ist Zeit fuer eine kleine Case Study, anhand der Policy aus Default1.thy *}

(* specific 
constdefs
    intranet::"IntegerPair subnet"
   "intranet \<equiv> {{(a,b) . (a > 1 \<and> a < 4) }}"

  dmz :: "IntegerPair subnet"
  "dmz \<equiv> {{(a,b). (a > 6) \<and> (a < 8)}}"

  internet :: "IntegerPair subnet"
  "internet \<equiv> {{(a,b).  \<not> ( (a > 1 \<and> a < 4) \<or> (a > 6) \<and> (a < 8))   }}"

constdefs 
  Intranet_DMZ :: "(IntegerPair,'b) Rule"
  "Intranet_DMZ \<equiv> (allow_prot_from_to smtp intranet dmz) ++ 
                       (allow_prot_from_to imap intranet dmz)"

  Intranet_Internet :: "(IntegerPair,'b) Rule"
  "Intranet_Internet \<equiv> allow_all_but_prot_from_to smtp intranet internet"
 
  DMZ_Intranet :: "(IntegerPair,'b) Rule"
  "DMZ_Intranet \<equiv> deny_all_from_to dmz intranet"

  DMZ_Internet :: "(IntegerPair,'b) Rule"
  "DMZ_Internet \<equiv> allow_prot_from_to smtp dmz internet"
     
  Internet_Intranet :: "(IntegerPair,'b) Rule"
  "Internet_Intranet \<equiv> deny_all_from_to internet intranet"
          
  Internet_DMZ :: "(IntegerPair,'b) Rule"
  "Internet_DMZ \<equiv> (allow_prot_from_to_port http internet dmz 80) ++ 
                       (allow_prot_from_to_port smtp internet dmz 25)"


constdefs 
  policy :: "(IntegerPair, DummyContent) Policy"
 "policy \<equiv> deny_all ++ 
                  DMZ_Internet ++ 
                  DMZ_Intranet ++ 
                  Intranet_Internet ++ 
                  Intranet_DMZ ++  
                  Internet_DMZ ++ 
                  Internet_Intranet"


lemmas PolicyLemmas =  policy_def 
                       DMZ_Internet_def DMZ_Intranet_def 
                       Intranet_Internet_def Intranet_DMZ_def 
                       Internet_DMZ_def Internet_Intranet_def


lemma "policy = X"
apply (unfold PolicyLemmas)
(* 
Die Policy besteht aus 9 Rules.
 Machen wir eine Schritt-fuer-Schritt Simplifikation: 
*)


(* Als erstes, aehnliche Rules zusammenfassen *)
apply (simp add: p2a1 p2a2 p2b1 p2b2 p2c1 p2c2)


(* Dann: die deny_all_from_to Rules sind unnoetig *)

(* Zuerst ein bisschen kommutieren \<dots> *)
apply (subst p3g)
defer 1
(* Jetzt das entsprechende Lemma anwenden. *)
apply (subst p1a)

(* Auch die deny_all_from_to Regel am Schluss ist unnoetig. Um sie wegzuwerfen, muss sie aber ganz nach vorne gebracht werden - und entsprechende disjunkt-assumptions bewiesen werden. *)
apply (subst p3f) back back back 
defer 1
apply (subst p3g) back back
defer 1
apply (subst p3g)  back
defer 1
apply (subst p3g) 
defer 1
apply (subst p1a)  

(* Hat auch geklappt. Die Policy wurde erfolgreich von 9 auf 5 Rules simplifiziert! 

Nun muessen aber noch die disjunkt-assumptions bewiesen werden.*)
defer 1
apply (simp add: dom_def dmz_def internet_def intranet_def IntegerPairCombinators src_def dest_def protocol_def in_subnet_def stateless_rules dest_port_def)
defer 1
apply (simp add: dom_def dmz_def internet_def intranet_def IntegerPairCombinators  src_def dest_def protocol_def in_subnet_def stateless_rules dest_port_def)
defer 1
apply (simp add: dom_def  dmz_def internet_def intranet_def IntegerPairCombinators  src_def dest_def protocol_def in_subnet_def stateless_rules dest_port_def)
defer 1
apply (simp add: dom_def  dmz_def internet_def intranet_def IntegerPairCombinators  src_def dest_def protocol_def in_subnet_def stateless_rules dest_port_def)
defer 1
apply (simp add: dom_def dmz_def internet_def intranet_def IntegerPairCombinators  src_def dest_def protocol_def in_subnet_def stateless_rules dest_port_def)
defer 1
apply auto
(* Hat geklappt, wir haben nun eine deutlich kleinere Policy! *)
oops

text {* 
Einige Beobachtungen: 

 - Die ersten Resultate sind ermutigend. Policies zu simplifizieren "funktioniert"
 - Eine vollstaendigere Theorie ist wuenschenswert.
 - Es sollte noch mehr automatisiert laufen  - ohne alle diese substs und backs. Momentan muss man fuer die erste Kategorie von Vereinfachungen mehr oder weniger sehen, was man wie simplifizieren will, und es dann einfach beweisen. 


 - Was waere noch moeglich? Noch andere Simplifikationen. Evtl. sogar eine Funktion, welche die Rules zuerst nach einem Schema ordnet und dann simplifiziert.

*}

*)


lemmas PSss = p1a p1b p1c p1d p1e p1f p1i p2a1 p2a2 p2a3


ML {*
val map_assoc = @{thm map_assoc}
val PSss = @{thms PSss}

*}

end

